from . import dam_break, sill
import numpy as np
import os

thisdir = os.path.dirname(__file__)
path_expected_sols = os.path.join(thisdir,'expected_sols.npy')
expected_sols_dict = np.load(path_expected_sols,allow_pickle=True).item()

def error(test_name,**kwargs):
    if "dambreak" in test_name:
        claw = dam_break.setup(**kwargs)
    elif "sill" in test_name:
        claw = sill.setup(**kwargs)
    else:
        raise ValueError(f"Test name {test_name} not recognized")
    claw.run()
    test_depth = claw.frames[claw.num_output_times].state.get_q_global()[0].reshape([-1])
    dx = claw.solution.domain.grid.delta[0]
    assert test_name in expected_sols_dict.keys(), f"Test name {test_name} not found in {path_expected_sols}"
    expected_depth = expected_sols_dict[test_name]
    diff_L1 = dx*np.sum(np.abs(test_depth-expected_depth))
    return diff_L1

class TestShallowWater1D:
    def test_dambreak(self):
        solver_types = ['classic','sharpclaw']
        riemann_solvers = ['hlle','roe']

        #Tests Fortran
        for solver_type in solver_types:
            for riemann_solver in riemann_solvers:
                test_name = f'dambreak_fortran_{solver_type}_{riemann_solver}'
                assert error(test_name=test_name, kernel_language="Fortran", solver_type=solver_type,
                            riemann_solver=riemann_solver, disable_output=True)<1e-6, f"Test {test_name} failed"
        #Tests Python
        for solver_type in solver_types:
            test_name = f'dambreak_python_{solver_type}'
            assert error(test_name=test_name, kernel_language="Python", solver_type=solver_type,
                         riemann_solver="hlle", disable_output=True)<1e-6, f"Test {test_name} failed"
        
    def test_sill(self):
        for kernel_language in ['Fortran','Python']:
            test_name = f'sill_{kernel_language}'
            assert error(test_name=test_name, kernel_language=kernel_language, outdir=None)<1e-6, f"Test {test_name} failed"
    
    


"""
Plot results at gauges to compare with Figure 5 of Matsuyama et al. (2007).
Blue curves are the GeoClaw results.

Wavetank data for some of the gauges is available in the file
data_wavegauge.csv (kindly provided by Prof. Matsuyama).
At these gauges the observations are also plotted as red curves.
"""

from pylab import *
import clawpack.pyclaw.gauges as gauges

outdir = '_output'

add_data = True


figure(400, figsize=(14,8))
clf()

xp_gauges = [-150, -80., -60, -50, -40, -32, -31.2, -30.8, -30, \
             -28, -20, -10, -5]
             
gauge_info = [(-80,   1,(10,60)),
              (-60,   3,(20,70)),
              (-50,   5,(30,80)),
              (-40,   7,(40,90)),
              (-32,   9,(40,90)),
              (-31.2,11,(40,90)),
              (-30.8, 2,(40,90)),
              (-30,   4,(40,90)),
              (-28,   6,(40,90)),
              (-20,   8,(50,100)),
              (-10,  10,(60,110)),
              (-5,   12,(60,110))]
          
                 
for info in gauge_info:
    xp_g, k, xlimits = info
    subplot(6,2,k)
    gaugeno = int(-xp_g*10)
    gauge = gauges.GaugeSolution(gaugeno, outdir)
    t = gauge.t
    eta = gauge.q[2,:]

    plot(t, 100*eta, 'b', label='x = %.1f' % xp_g)

    ylim(-5, 15)
    xlim(xlimits)
    grid(True)
    yticks([-5,0,5,10,15])
    xlabel('')
    #ylabel('Surface (m)')
    #title('Gauge %i' % gaugeno)

    if k != 2:
        legend(loc='upper right',framealpha=1)

if add_data:
    d = loadtxt('data_wavegauge.csv',skiprows=2,delimiter=',')
    subplot(6,2,5)
    plot(d[:,0],d[:,2],'r')
    subplot(6,2,7)
    plot(d[:,0],d[:,3],'r')
    subplot(6,2,2)
    plot(d[:,0],d[:,4],'r',label='observed')
    legend(loc='upper right',framealpha=1)
    subplot(6,2,8)
    plot(d[:,0],d[:,5],'r')
    subplot(6,2,12)
    plot(d[:,0],d[:,6],'r')

tight_layout()

if 1:
    fname = 'GaugesFigure5.png'
    savefig(fname, bbox_inches='tight')
    print('Created %s' % fname)



#!/usr/bin/env python

import sys

import numpy
import matplotlib.pyplot as plt

import clawpack.pyclaw.gauges as gauges

# Load all gauges

def check_old_gauge_data(path, gauge_id):

    # Load old gauge data
    data = numpy.loadtxt(path)
    old_ids = numpy.asarray(data[:, 0], dtype=int)
    gauge_indices = numpy.nonzero(old_ids == gauge_id)[0]
    q = data[gauge_indices, 3:]

    # Load new data
    gauge = gauges.GaugeSolution(gauge_id, "./regression_data/")

    print(numpy.linalg.norm(q - gauge.q.transpose(), ord=2))
    print(numpy.argmax(q - gauge.q.transpose()))

    fig = plt.figure()
    for i in range(gauge.q.shape[0]):
        axes = fig.add_subplot(1, gauge.q.shape[0], i + 1)
        axes.plot(q[:, i] - gauge.q[i, :])
        axes.set_title("q[%s, :] comparison" % i)

    return fig

if __name__ == "__main__":

    old_files = []
    gauge_ids = []

    if len(sys.argv) > 1:
        # Assume arguments are pairs of file locations and ids
        for i in range(1, len(sys.argv), 2):
            old_files.append(sys.argv[i])
            gauge_ids.append(int(sys.argv[i + 1]))
    else:
        raise ValueError("Need at least one pair to compare")

    print(old_files)
    print(gauge_ids)

    figures = []
    for i in range(len(old_files)):
        figures.append(check_old_gauge_data(old_files[i], gauge_ids[i]))

    plt.show()


#!/usr/bin/env python
# encoding: utf-8
"""
Generic code for running the fortran version of Clawpack and sending the
results to subdirectory output of the directory from which this is executed.
Execute via
    $ python $CLAW/clawutil/src/python/clawutil/runclaw.py
from a directory that contains a claw.data file and a Clawpack executable.
"""

import os
import sys
import glob
import shutil
import shlex
import subprocess
import time
import warnings
import runpy

from clawpack.clawutil.data import ClawData
from clawpack.clawutil.claw_git_status import make_git_status_file

# define an execution error class that returns a
# message as well as the rest of the subprocess exceptions
class ClawExeError(subprocess.CalledProcessError):
    def __init__(self, msg, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.msg = msg
        
    def __str__(self):
        return self.msg

def runclaw(xclawcmd=None, outdir=None, overwrite=True, restart=None, 
            rundir=None, print_git_status=False, nohup=False, nice=None,
            runexe=None,
            xclawout=None, xclawerr=None, verbose=True):
    """
    Run the Fortran version of Clawpack using executable xclawcmd, which is
    typically set to 'xclaw', 'xamr', etc.

    If it is not set by the call, get it from the environment variable
    CLAW_EXE.  Default to 'xclaw' if that's not set.

    If overwrite is True, it's ok to zap current contents of the outdir.
    If overwrite is False, move the outdir (or copy in the case of a restart)
    to a backup directory with a unique name based on time executed.

    If restart is None, determine whether this is a restart from claw.data
    (as set in setrun.py).  Can remove setting RESTART in Makefiles.
    
    If rundir is None, all *.data is copied from current directory, if a path 
    is given, data files are copied from there instead.

    If print_git_status is True, print a summary of the git status of all
    clawpack repositories in the file claw_git_status.txt in outdir.

    If nohup is True, run the xclawcmd in nohup mode:  runs in background and
    output goes into nohup.out file in the directory specified by outdir,
    rather than all output going to the screen.  The job keeps running if user 
    logs off.  Useful for scripts starting long jobs in batch mode.

    If type(nice) is int, runs the code using "nice -n "
    with this nice value so it doesn't hog computer resources.
    
    xclawout and xclawerr define the locations of stdout and stderr for the 
    execution of CLAW_EXE. They should be strings to filepaths or open file
    objects or ``subprocess.PIPE`` or ``subprocess.STDOUT``. To pass both
    to the same file, specify ``xclawout`` as the filepath and 
    ``xclawerr=subprocess.STDOUT``.

    """
    
    if nice is not None:
        try:
            nice = int(nice)
        except:
            nice = None

    # convert strings passed in from Makefile to boolean:
    if type(overwrite) is str:
        overwrite = (overwrite.lower() in ['true','t'])
    if type(restart) is str:
        if restart == 'None':
            restart = None
        else:
            restart = (restart.lower() in ['true','t'])
    if type(print_git_status) is str:
        print_git_status = (print_git_status.lower() in ['true','t'])
    if type(nohup) is str:
        nohup = (nohup.lower() in ['true','t'])
    

    if xclawcmd is None:
        # Determine what executable to use from environment variable CLAW_EXE
        # Default to 'xclaw' if it's not set:
        xclawcmd = os.environ.get('CLAW_EXE', 'xclaw')
    

    if outdir is None:
        outdir = '.'

    if rundir is None:
        rundir = os.getcwd()
    rundir = os.path.abspath(rundir)
    print("==> runclaw: Will take data from ", rundir)

    # directory for fort.* files:
    outdir = os.path.abspath(outdir)
    print('==> runclaw: Will write output to ',outdir)
    
    if restart is None:
        # Added option to determine restart from claw.data (i.e. setrun.py)
        clawdata = ClawData()
        clawdata.read(os.path.join(rundir,'claw.data'), force=True) 
        restart = clawdata.restart
        
    xclawcmd = os.path.abspath(xclawcmd)

    if os.path.isfile(outdir):
        print("==> runclaw: Error: outdir specified is a file")
        return

    if (os.path.isdir(outdir) & (not overwrite)):
        # copy the old outdir before possibly overwriting
        tm = time.localtime(os.path.getmtime(outdir))
        year = str(tm[0]).zfill(4)
        month = str(tm[1]).zfill(2)
        day = str(tm[2]).zfill(2)
        hour = str(tm[3]).zfill(2)
        minute = str(tm[4]).zfill(2)
        second = str(tm[5]).zfill(2)
        outdir_backup = outdir + '_%s-%s-%s-%s%s%s' \
              % (year,month,day,hour,minute,second)
        if verbose:
            print("==> runclaw: Directory already exists: ",os.path.split(outdir)[1])
            if restart:
                print("==> runclaw: Copying directory to:      ",os.path.split(outdir_backup)[1])
            else:
                print("==> runclaw: Moving directory to:      ",os.path.split(outdir_backup)[1])
            time.sleep(1)

        try:
            shutil.move(outdir,outdir_backup)
            if restart:
                shutil.copytree(outdir_backup,outdir)
        except:
            print("==> runclaw: Could not move directory... copy already exists?")

    os.makedirs(outdir, exist_ok=True)

    if print_git_status not in [False,'False']:
        # create files claw_git_status.txt and claw_git_diffs.txt in
        # outdir:
        make_git_status_file(outdir=outdir)

    # old fort.* files to be removed for new run?
    fortfiles = glob.glob(os.path.join(outdir,'fort.*'))
    # also need to remove gauge*.txt output files now that the gauge
    # output is no longer in fort.gauge  (but don't remove new gauges.data)
    gaugefiles = glob.glob(os.path.join(outdir,'gauge*.txt'))

    if (overwrite and (not restart)):
        # remove any old versions:
        if verbose:
            print("==> runclaw: Removing all old fort/gauge files in ", outdir)
        for file in fortfiles + gaugefiles:
            os.remove(file)
    elif restart:
        if verbose:
            print("==> runclaw: Restart: leaving original fort/gauge files in ", outdir)
    else:
        # this should never be reached: 
        # if overwrite==False then outdir has already been moved
        if len(fortfiles+gaugefiles) > 1:
            print("==> runclaw: *** Remove fort.* and gauge*.txt")
            print("  from output directory %s and try again," % outdir)
            print("  or use overwrite=True in call to runclaw")
            print("  e.g., by setting OVERWRITE = True in Makefile")
            return

    datafiles = glob.glob(os.path.join(rundir,'*.data'))
    if datafiles == ():
        print("==> runclaw: Warning: no data files found in directory ",rundir)
    else:
        if rundir != outdir:
            for file in datafiles:
                shutil.copy(file, os.path.join(outdir,os.path.basename(file)))

    b4run = None
    if os.path.isfile('b4run.py'):
        b4run_file = os.path.abspath('b4run.py')
    else:
        b4run_file = os.environ.get('B4RUN', '')

    if os.path.isfile(b4run_file):
        try:
            file_globals = runpy.run_path(b4run_file)
            b4run = file_globals.get('b4run', None)
        except:
            w = r"*** WARNING: problem running b4run_file = %s" % b4run_file
            warnings.warn(w, UserWarning)
            b4run = None
        
    if b4run is not None:
        try:
            b4run(rundir, outdir)
            print('Executed b4run function from %s' % b4run_file)
        except:
            w = r"*** WARNING: problem executing b4run from %s" % b4run_file
            warnings.warn(w, UserWarning)

    # execute command to run fortran program:

    if runexe is not None:
        xclawcmd = runexe + ' ' + xclawcmd

    if nohup:
        # run in nohup mode:
        print("\n==> Running in nohup mode, output will be sent to:")
        print("      %s/nohup.out" % outdir)
        if type(nice) is int:
            cmd = "nohup time nice -n %s %s " % (nice,xclawcmd)
        else:
            cmd = "nohup time %s " % xclawcmd
        print("\n==> Running with command:\n   ", cmd)
    else:
        if type(nice) is int:
            cmd = "nice -n %s %s " % (nice,xclawcmd)
        else:
            cmd = xclawcmd
        print("\n==> Running with command:\n   ", cmd)

    cmd_split = shlex.split(cmd)
    if isinstance(xclawout, str):
        xclawout = open(xclawout,'w', encoding='utf-8',
                        buffering=1)
    if isinstance(xclawerr, str):
        xclawerr = open(xclawerr,'w', encoding='utf-8',
                        buffering=1)
    try:
        proc = subprocess.check_call(cmd_split,
                                     cwd=outdir,
                                     stdout=xclawout,
                                     stderr=xclawerr)

    except subprocess.CalledProcessError as cpe:
        exe_error_str = "\n\n*** FORTRAN EXE FAILED ***\n"
        raise ClawExeError(exe_error_str, cpe.returncode, cpe.cmd,
                           output=cpe.output, 
                           stderr=cpe.stderr)
    
    print('==> runclaw: Done executing %s via clawutil.runclaw.py' %\
                xclawcmd)
    print('==> runclaw: Output is in ', outdir)

    return proc
    

#----------------------------------------------------------

def create_path(path,overwrite=False):
    r"""Create a path and over write it if requested"""
    
    if not os.path.exists(path):
        os.makedirs(path)
    elif overwrite:
        data_files = glob.glob(path,"*")
        for data_file in data_files:
            os.remove(data_file)

def create_output_paths(name,prefix,**kargs):
    r"""Create an output, plotting, and data path with the given prefix."""
    
    if 'outdir' in kargs:
        outdir = kargs['outdir']
    else:
        base_path = os.environ.get('DATA_PATH',os.getcwd())
        outdir = os.path.join(base_path,name,"%s_output" % prefix)
    if 'plotdir' in kargs:
        plotdir = kargs['plotdir']
    else:
        base_path = os.environ.get('DATA_PATH',os.getcwd())
        plotdir = os.path.join(base_path,name,"%s_plots" % prefix)
    if 'logfile' in kargs:
        log_path = kargs['logfile']
    else:
        base_path = os.environ.get('DATA_PATH',os.getcwd())
        log_path = os.path.join(base_path,name,"%s_log.txt" % prefix)
        
    create_path(outdir,overwrite=kargs.get('overwrite',False))
    create_path(plotdir,overwrite=kargs.get('overwrite',False))
    create_path(os.path.dirname(log_path),overwrite=False)
    
    return outdir,plotdir,log_path


def replace_stream_handlers(logger_name,log_path,log_file_append=True):
    r"""Replace the stream handlers in the logger logger_name
        
        This routine replaces all stream handlers in logger logger_name with a 
        file handler which outputs to the log file at log_path. If 
        log_file_append is True then the log files is opened for appending, 
        otherwise it is written over.
        
        TODO: This seems to only be working in certain instances, need to debug
    """
    import logging
    
    logger = logging.getLogger(logger_name)
    handler_list = [handler for handler in logger.handlers if isinstance(handler,logging.StreamHandler)]
    for handler in handler_list:
        # Create new handler
        if log_file_append:
            new_handler = logging.FileHandler(log_path,'a')
        else:
            new_handler = logging.FileHandler(log_path,'w')
            log_file_append = True
        new_handler.setLevel(handler.level)
        # new_handler.name = handler.name
        new_handler.setFormatter(handler.formatter)
            
        # Remove old handler
        if isinstance(handler,logging.FileHandler):
            handler.close()
            if os.path.exists(handler.baseFilename):
                os.remove(handler.baseFilename)
        logger.removeHandler(handler)
          
        # Add new handler  
        logger.addHandler(new_handler)


if __name__=='__main__':
    """
    If executed at command line prompt, simply call the function, with
    any argument used as setplot:
    """
    import sys
    args = sys.argv[1:]   # any command line arguments
    runclaw(*args)




"""
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.

"""

import numpy as np
import matplotlib.pyplot as plt

from clawpack.geoclaw import topotools, kmltools

try:
    TG32412 = np.loadtxt('32412_notide.txt')
except:
    print("*** Could not load DART data file")

#--------------------------
def setplot(plotdata):
#--------------------------

    """
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.

    """


    from clawpack.visclaw import colormaps, geoplot
    from numpy import linspace

    plotdata.clearfigures()  # clear any old figures,axes,items data

    plotdata.verbose = False

    # To plot gauge locations on pcolor or contour plot, use this as
    # an afteraxis function:

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos='all', format_string='ko', add_labels=True)


    #-----------------------------------------
    # Some global kml flags
    #-----------------------------------------
    plotdata.kml_name = "Chile 2010"
    plotdata.kml_starttime = [2010,2,27,6,34,0]  # Time of event in UTC [None]
    plotdata.kml_tz_offset = 3    # Time zone offset (in hours) of event. [None]

    plotdata.kml_index_fname = "Chile_2010"  # name for .kmz and .kml files ["_GoogleEarth"]

    # Set to a URL where KMZ file will be published.
    # plotdata.kml_publish = None

    # Colormap range
    cmin = -0.2
    cmax = 0.2
    cmap = geoplot.googleearth_transparent

    #-----------------------------------------------------------
    # Figure for KML files
    #----------------------------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Sea Surface',figno=1)
    plotfigure.show = True

    plotfigure.use_for_kml = True
    plotfigure.kml_use_for_initial_view = True
    plotfigure.kml_show_figure = True

    # These override any axes limits set below in plotaxes
    plotfigure.kml_xlimits = [-120,-60]
    plotfigure.kml_ylimits = [-60, 0.0];

    # Resolution (should be consistent with data)
    # Refinement levels : [2,6]; max level = 3; num_cells = [30,30]
    # rcl : resolution of the coarsest level in this figure
    rcl = 5    # choose so rcl*figsize = integer multiple of num_cells
    plotfigure.kml_figsize = [30.0,30.0]
    plotfigure.kml_dpi = rcl*2*6         # Resolve all three levels
    plotfigure.kml_tile_images = False    # Tile images for faster loading.  Requires GDAL


    # Water
    plotaxes = plotfigure.new_plotaxes('kml')
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = cmap
    plotitem.pcolor_cmin = cmin
    plotitem.pcolor_cmax = cmax

    def kml_colorbar(filename):
        kmltools.kml_build_colorbar(filename,cmap=cmap,cmin=cmin,cmax=cmax,
                                    label='meters', title='surface',
                                    extend='both')

    plotfigure.kml_colorbar = kml_colorbar

    #-----------------------------------------------------------
    # Figure for KML files (zoom)
    #----------------------------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Sea Surface (zoom)',figno=2)
    #plotfigure.show = False

    plotfigure.use_for_kml = True
    plotfigure.kml_use_for_initial_view = False  # Use large plot for view
    plotfigure.kml_show_figure = True

    # Set Google Earth bounding box and figure size
    plotfigure.kml_xlimits = [-84,-74]
    plotfigure.kml_ylimits = [-18,-4]
    plotfigure.kml_figsize = [10,14]  # inches.

    # Resolution
    rcl = 10    # Over-resolve the coarsest level
    plotfigure.kml_dpi = rcl*2*6       # Resolve all three levels
    plotfigure.kml_tile_images = False  # Tile images for faster loading.


    plotaxes = plotfigure.new_plotaxes('kml')
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = cmap
    plotitem.pcolor_cmin = cmin
    plotitem.pcolor_cmax = cmax


    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface at gauges', figno=300, \
                    type='each_gauge')
    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    # plotaxes.title = 'Surface'

    # Plot surface as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 3
    plotitem.plotstyle = 'b-'

    # Plot topo as green curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.show = True

    def gaugetopo(current_data):
        q = current_data.q
        h = q[0,:]
        eta = q[3,:]
        topo = eta - h
        return topo

    plotitem.plot_var = gaugetopo
    plotitem.plotstyle = 'g-'

    def add_zeroline(current_data):
        from pylab import plot, legend, xticks, floor, axis, xlabel
        t = current_data.t
        gaugeno = current_data.gaugeno

        if gaugeno == 32412:
            try:
                plot(TG32412[:,0], TG32412[:,1], 'r')
                legend(['GeoClaw','Obs'],loc='lower right')
            except: pass
            axis((0,t.max(),-0.3,0.3))

        plot(t, 0*t, 'k')
        n = int(floor(t.max()/3600.) + 2)
        xticks([3600*i for i in range(n)], ['%i' % i for i in range(n)])
        xlabel('time (hours)')

    plotaxes.afteraxes = add_zeroline


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'         # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'           # list of figures to print
    plotdata.html = False                     # create html files of plots?
    plotdata.html_movie = None                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = False                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    plotdata.kml = True

    return plotdata




"""
Module to create topo and qinit data files for this example.
"""

from __future__ import absolute_import
from clawpack.geoclaw.topotools import Topography
from numpy import *

#from pyclaw.data import Data
#probdata = Data('setprob.data')

a = 1.
sigma = 0.5
h0 = 0.1
grav = 9.81
omega = sqrt(2.*grav*h0) / a

def maketopo():
    """
    Output topography file for the entire domain
    """
    nxpoints=200
    nypoints=200
    xupper=2.e0
    yupper=2.e0
    xlower = -2.e0
    ylower = -2.e0
    outfile= "bowl.topotype2"

    topography = Topography(topo_func=topo)
    topography.x = linspace(xlower,xupper,nxpoints)
    topography.y = linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=2, Z_format="%22.15e")


def topo(x,y):
    """
    Parabolic bowl
    """
    z = h0*(x**2 + y**2)/a**2 - h0
    return z


if __name__=='__main__':
    maketopo()




# OLD:

    #------------------------------------------------------------------
    # Problem-specific parameters to be written to setprob.data:
    #------------------------------------------------------------------

    probdata = rundata.new_UserData(name='probdata',fname='setprob.data')
    probdata.add_param('minLevelBouss', 1,' minlevel for Bouss terms')
    probdata.add_param('maxLevelBouss', 10,' maxlevel for Bouss terms')
    probdata.add_param('deepBouss', 5,' min water depth for Bouss terms')
    probdata.add_param('solver', 3,     ' 1=GMRES, 2=Pardiso, 3=PETSc')
    probdata.add_param('equations', 2,     ' 1=MadsenSchaffer, 2=SGN')
    probdata.add_param('alpha', 1.153,  ' If using SGN, else ignore')
    probdata.add_param('startWithBouss', True, 'Take numSWEsteps of SWE first')
    probdata.add_param('numSWEsteps', 0,  ' Take this many SWE steps first')


# NEW:

    # To use Boussinesq solver, add bouss_data parameters here
    # Also make sure to use the correct Makefile pointing to bouss version
    from clawpack.geoclaw.data import BoussData
    rundata.add_data(BoussData(),'bouss_data')
    
    # CHECK ORDER!

    rundata.bouss_data.bouss_equations = 2    # 0=SWE, 1=MS, 2=SGN
    rundata.bouss_data.bouss_min_level = 1    # coarsest level to apply bouss
    rundata.bouss_data.bouss_max_level = 10   # finest level to apply bouss
    rundata.bouss_data.bouss_min_depth = 10.  # depth to switch to SWE
    rundata.bouss_data.bouss_solver = 3       # 1=GMRES, 2=Pardiso, 3=PETSc
    rundata.bouss_data.bouss_tstart = 0.      # time to switch from SWE



import glob

try:
    import rioxarray
    import xarray as xr
except ImportError:
    "You must install xarray and rioxarray in order to use the xarray backends"
    raise

from clawpack.geoclaw.xarray_backends import FGMaxBackend, FGOutBackend

# epsg code for lat-lon
# Optionally, provide an epsg code to assign the associated coordinate system to the file.
# default behavior assigns no coordinate system. Note that if no coordinate system is provided,
# it will come in a GIS with coordinates associated with row and column number (not the x and y position
# encoded in the netcdf).

epsg_code = 4326

# An example of a fgout grid.
filename = "_output/fgout0001.b0001"
# provide the .bxxx file if binary format is used or the
# .qxxx file if ascii format is used.
# the format, fg number, and frame number are inferred from the filename.

ds = xr.open_dataset(
    filename,
    engine=FGOutBackend,
    backend_kwargs={
        "epsg": epsg_code,
        "qmap": "geoclaw",
        # qmap is the qmap specified to the fgout object in setrun.py see
        # the following documentation page for more details.
        # https://www.clawpack.org/dev/fgout.html#specifying-q-out-vars
        "dry_tolerance": None,
        # variables that are not eta and B are masked
        # where h>dry_tolerance. To turn this functionality
        # off set dry_tolerance = None.
    },
)
# ds is now an xarray object. It can be interacted with directly or written to netcdf using
ds.to_netcdf("fgout0001_0001.nc")

# It is possible to combine all fgout files into a single netcdf file
# using xr.open_mfdataset (requires dask) or xr.concat (does not require dask)
# https://docs.xarray.dev/en/stable/generated/xarray.open_mfdataset.html
# https://docs.xarray.dev/en/latest/generated/xarray.concat.html
# for instructions on installing xarray with dask see:
# https://docs.xarray.dev/en/latest/getting-started-guide/installing.html#instructions

fgout_files = glob.glob("_output/fgout0001.b*")

try:
    ds_all = xr.open_mfdataset(
        fgout_files,
        engine=FGOutBackend,
        backend_kwargs={"epsg": epsg_code, "qmap": "geoclaw"},
    )
except ValueError:  # if dask is not available, use xr.concat.
# if dask is not installed xr.open_mfdataset() will fail with something like
# ValueError: unrecognized chunk manager dask - must be one of: []
    fgouts = []
    for filename in fgout_files:
        ds = xr.open_dataset(
            filename,
            engine=FGOutBackend,
            backend_kwargs={"epsg": epsg_code, "qmap": "geoclaw"},
        )
        fgouts.append(ds)

    ds_all = xr.concat(fgouts, dim="time")

# save out.
ds_all.to_netcdf("fgout_all.nc")

# An example of a fgmax grid.
filename = "_output/fgmax0001.txt"
ds = xr.open_dataset(filename, engine=FGMaxBackend, backend_kwargs={"epsg": epsg_code})
ds.to_netcdf("fgmax0001.nc")

# To see the use of clipping, change the tfinal in setrun to something like 2*3600.0
# the fgmax0001_clipped.nc will only be the area where the wave arrived within the considered time.
filename = "_output/fgmax0001.txt"
ds = xr.open_dataset(
    filename, engine=FGMaxBackend, backend_kwargs={"epsg": epsg_code, "clip": True}
)
ds.to_netcdf("fgmax0001_clipped.nc")



#!/usr/bin/env python

# How to use this file
# python setup.py build_ext -i

import os
from os.path import join as pjoin
from os.path import pardir as pardir

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('shallow_sphere', parent_package, top_path)

    if top_path:
        # installing into clawpack
        classic_dir = pjoin(top_path, 'pyclaw', 'src', 'pyclaw', 'classic')
    else:
        # building as part of repository checkout
        this_dir = os.path.dirname(os.path.realpath(__file__))
        classic_dir = pjoin(this_dir, pardir, pardir, 'src', 'pyclaw', 'classic')
    classic_srcs = [pjoin(classic_dir, src) for src in ['limiter.f90',
                                                        'philim.f90',
                                                        'flux2.f90',
                                                        'step2ds.f90']]

    config.add_extension('classic2',
                         ['step2qcor.f90','qcor.f90'] + classic_srcs)

    config.add_extension('problem',
                         ['mapc2p.f90','setaux.f90','qinit.f90','src2.f90'])

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



# We don't compile the layered shallow water solver by default.
# To compile it, do
#
# f2py -c ../../rp1_layered_shallow_water.f90 -m layered_shallow_water_1D

one_d_ptwise_riemann = ['acoustics',
                        'advection']

one_d_riemann = ['acoustics',
                 'acoustics_variable',
                 'advection',
                 'advection_color',
                 'burgers',
                 'cubic',
                 'traffic',
                 'traffic_vc',
                 'traffic_vc_fwave',
                 'traffic_vc_tracer',
                 'euler_with_efix',
                 'euler_hlle',
                 'mhd_roe',
                 'nonlinear_elasticity_fwave',
                 'psystem_fwave',
                 'reactive_euler_with_efix',
                 'shallow_hlle',
                 'shallow_roe_with_efix',
                 'shallow_bathymetry_fwave',
                 'shallow_roe_tracer']

two_d_ptwise_riemann = ['acoustics']

two_d_riemann = ['acoustics',
                 'acoustics_mapped',
                 'advection',
                 'burgers',
                 'euler_5wave',
                 'psystem',
                 'shallow_roe_with_efix',
                 'shallow_bathymetry_fwave',
                 'shallow_sphere',
                 'vc_acoustics',
                 'vc_advection',
                 'vc_elasticity']

three_d_riemann = ['vc_acoustics',
                   'euler',
                   'burgers',
                   'vc_advection']

# special rules for rp2_kpp, rp2_euler_mapgrid

import os

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('riemann', parent_package, top_path)

    src_dir = os.path.join(os.path.dirname(__file__),os.path.join('..','src'))

    for rp in one_d_ptwise_riemann:
        rp_ext = rp + "_1D_ptwise"
        rp_src = [os.path.join(src_dir, 'rp1_ptwise.f90'),
                  os.path.join(src_dir, 'rp1_' + rp + '_ptwise.f90')]
        config.add_extension(rp_ext, rp_src)

    for rp in one_d_riemann:
        rp_ext = rp + '_1D'
        rp_src = [os.path.join(src_dir, 'rp1_' + rp + '.f90')]
        config.add_extension(rp_ext, rp_src)

    for rp in two_d_ptwise_riemann:
        rp_ext = rp + "_2D_ptwise"
        rp_src = []
        for prefix in ['rpn2_', 'rpt2_']:
            rp_src.append(os.path.join(src_dir, prefix + 'ptwise.f90'))
            rp_src.append(os.path.join(src_dir, prefix + rp + '_ptwise.f90'))
        config.add_extension(rp_ext, rp_src)

    for rp in two_d_riemann:
        rp_ext = rp + '_2D'
        rp_src = [os.path.join(src_dir, prefix + rp + '.f90')
                  for prefix in ['rpn2_', 'rpt2_']]
        config.add_extension(rp_ext, rp_src)

    for rp in three_d_riemann:
        rp_ext = rp + '_3D'
        rp_src = [os.path.join(src_dir, prefix + rp + '.f90')
                  for prefix in ['rpn3_', 'rpt3_', 'rptt3_']]
        config.add_extension(rp_ext,rp_src)

    # special targets
    special_target_list = \
    [{'ext' :'kpp_2D',
      'srcs':['rpn2_kpp.f90','rpt2_dummy.f90']},
     {'ext' :'shallow_hlle_2D',
      'srcs':['rpn2_shallow_hlle.f90','rpt2_dummy.f90']},
     {'ext' :'euler_hlle_2D',
      'srcs':['rpn2_euler_hlle.f90','rpt2_dummy.f90']},
     {'ext' :'euler_hlle_with_walls_2D',
      'srcs':['rpn2_euler_hlle_with_walls.f90','rpt2_dummy.f90']},
     {'ext' :'euler_mapgrid_2D',
      'srcs':['rpn2_euler_mapgrid.f90','rpt2_euler_mapgrid.f90',
              'euler_roe_solver_mapgrid.f90','getquadinfo_mapgrid.f90']},
     {'ext' :'euler_4wave_2D',
      'srcs':['rpn2_euler_4wave.f90','rpt2_euler.f90']}]

    for rp_dict in special_target_list:
        rp_ext = rp_dict['ext']
        rp_src = [os.path.join(src_dir,src) for src in rp_dict['srcs']]

        config.add_extension(rp_ext,rp_src)

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python

# How to use this file
# python setup.py build_ext -i

import os
from os.path import join as pjoin
from os.path import pardir as pardir

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('euler_1d', parent_package, top_path)

    if top_path:
        # installing into clawpack
        sharpclaw_dir = pjoin(top_path, 'pyclaw', 'src', 'pyclaw', 'sharpclaw')
    else:
        # building as part of repository checkout
        this_dir = os.path.dirname(os.path.realpath(__file__))
        sharpclaw_dir = pjoin(this_dir, pardir, pardir, 'src', 'pyclaw', 'sharpclaw')
    sharpclaw_srcs = [pjoin(sharpclaw_dir, src) for src in ['ClawParams.f90',
                                                            'workspace.f90',
                                                            'weno.f90',
                                                            'reconstruct.f90','flux1.f90']]

    config.add_extension('sharpclaw1',
                         ['evec.f90'] + sharpclaw_srcs)
    config.add_extension('euler_tfluct','euler_tfluct.f90')

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python

import datetime
import os.path
import shutil
import tempfile

from nose.tools import raises
import numpy as np

import clawpack.geoclaw.util as util
from clawpack.geoclaw.util import NOAA_API_URL, fetch_noaa_tide_data


class TestFetchNoaaTideData:
    station = '1234567'
    begin_date = datetime.datetime(2000, 10, 30, 12, 0)
    end_date = datetime.datetime(2000, 10, 30, 12, 24)

    water_level_url = '{}?product={}&station={}'.format(
        NOAA_API_URL, 'water_level', station)
    predictions_url = '{}?product={}&station={}'.format(
        NOAA_API_URL, 'predictions', station)

    @classmethod
    def setup_class(cls):
        cls.temp_dir = tempfile.mkdtemp()

    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.temp_dir)

    def test_retrieve_and_cache(self):
        cache_dir = os.path.join(self.temp_dir,
                                 self.test_retrieve_and_cache.__name__)

        water_level_response = \
            ('Date Time, Water Level, Sigma, O or I (for verified), F, R, L, Quality\n'
             '2000-10-30 12:00,1.001,0.001,0,0,0,0,v\n'
             '2000-10-30 12:06,1.002,0.002,0,0,0,0,v\n'
             '2000-10-30 12:12,1.003,0.003,0,0,0,0,v\n'
             '2000-10-30 12:18,1.004,0.004,0,0,0,0,v\n'
             '2000-10-30 12:24,1.005,0.005,0,0,0,0,v\n')

        predictions_response = ('Date Time, Prediction\n'
                                '2000-10-30 12:00,1.101\n'
                                '2000-10-30 12:06,1.102\n'
                                '2000-10-30 12:12,1.103\n'
                                '2000-10-30 12:18,1.104\n'
                                '2000-10-30 12:24,1.105\n')

        date_time_range = ['2000-10-30T12:00',
                           '2000-10-30T12:06',
                           '2000-10-30T12:12',
                           '2000-10-30T12:18',
                           '2000-10-30T12:24']

        date_time_expected = np.array(date_time_range, dtype='datetime64[m]')
        water_level_expected = np.array([1.001, 1.002, 1.003, 1.004, 1.005])
        prediction_expected = np.array([1.101, 1.102, 1.103, 1.104, 1.105])

        # monkey patch urllib to return mock data
        def mock_read_response(url):
            if 'product=water_level' in url:
                return water_level_response
            if 'product=predictions' in url:
                return predictions_response
            raise AssertionError
        self._monkey_patch_urlopen(mock_read_response)

        # first time, should fetch data and save in cache
        self._fetch_and_assert(date_time_expected, water_level_expected,
                               prediction_expected, cache_dir)

        # configure endpoint to not return data anymore
        self._monkey_patch_urlopen(lambda url: 'Should read from cache')

        # second time, should read data from cache
        self._fetch_and_assert(date_time_expected, water_level_expected,
                               prediction_expected, cache_dir)

    def test_api_error(self):
        cache_dir = os.path.join(self.temp_dir, self.test_api_error.__name__)

        # configure endpoint to return an error
        self._monkey_patch_urlopen(lambda url: 'Something went wrong')

        # should return None
        d,w,p = fetch_noaa_tide_data(self.station, self.begin_date, self.end_date,
                             cache_dir=cache_dir)
        assert d == None, '*** expected d == None'

    @raises(ValueError)
    def test_date_time_range_mismatch(self):
        cache_dir = os.path.join(self.temp_dir,
                                 self.test_date_time_range_mismatch.__name__)

        # missing first two entries
        water_level_response = \
            ('Date Time, Water Level, Sigma, O or I (for verified), F, R, L, Quality\n'
             '2000-10-30 12:12,1.003,0.003,0,0,0,0,v\n'
             '2000-10-30 12:18,1.004,0.004,0,0,0,0,v\n'
             '2000-10-30 12:24,1.005,0.005,0,0,0,0,v\n')

        # missing last two entries
        predictions_response = ('Date Time, Prediction\n'
                                '2000-10-30 12:00,1.101\n'
                                '2000-10-30 12:06,1.102\n'
                                '2000-10-30 12:12,1.103\n')

        # monkey patch urllib to return mock data
        def mock_read_response(url):
            if 'product=water_level' in url:
                return water_level_response
            if 'product=predictions' in url:
                return predictions_response
            raise AssertionError
        self._monkey_patch_urlopen(mock_read_response)

        # should raise ValueError
        fetch_noaa_tide_data(self.station, self.begin_date, self.end_date,
                             cache_dir=cache_dir)

    def test_missing_values(self):
        cache_dir = os.path.join(self.temp_dir,
                                 self.test_missing_values.__name__)

        # missing fist two water level values
        water_level_response = \
            ('Date Time, Water Level, Sigma, O or I (for verified), F, R, L, Quality\n'
             '2000-10-30 12:00,,0.001,0,0,0,0,v\n'
             '2000-10-30 12:06,,0.002,0,0,0,0,v\n'
             '2000-10-30 12:12,1.003,0.003,0,0,0,0,v\n'
             '2000-10-30 12:18,1.004,0.004,0,0,0,0,v\n'
             '2000-10-30 12:24,1.005,0.005,0,0,0,0,v\n')

        # missing a prediction value
        predictions_response = ('Date Time, Prediction\n'
                                '2000-10-30 12:00,1.101\n'
                                '2000-10-30 12:06,1.102\n'
                                '2000-10-30 12:12,1.103\n'
                                '2000-10-30 12:18,\n'
                                '2000-10-30 12:24,1.105\n')

        date_time_range = ['2000-10-30T12:00',
                           '2000-10-30T12:06',
                           '2000-10-30T12:12',
                           '2000-10-30T12:18',
                           '2000-10-30T12:24']

        date_time_expected = np.array(date_time_range, dtype='datetime64[m]')
        water_level_expected = np.array([np.nan, np.nan, 1.003, 1.004, 1.005])
        prediction_expected = np.array([1.101, 1.102, 1.103, np.nan, 1.105])

        # monkey patch urllib to return mock data
        def mock_read_response(url):
            if 'product=water_level' in url:
                return water_level_response
            if 'product=predictions' in url:
                return predictions_response
            raise AssertionError
        self._monkey_patch_urlopen(mock_read_response)

        self._fetch_and_assert(date_time_expected, water_level_expected,
                               prediction_expected, cache_dir)

    def _fetch_and_assert(self, date_time_expected, water_level_expected,
                          prediction_expected, cache_dir):
        # fetch data
        date_time, water_level, prediction = fetch_noaa_tide_data(
            self.station, self.begin_date, self.end_date, cache_dir=cache_dir)

        # make sure data was correctly retrieved
        np.testing.assert_equal(date_time, date_time_expected)
        np.testing.assert_equal(water_level, water_level_expected)
        np.testing.assert_equal(prediction, prediction_expected)

    @staticmethod
    def _monkey_patch_urlopen(mock_read_response):
        """Modify 'util.urlopen' to return a custom response."""
        def mock_urlopen(url):
            class MockHttpResponse:
                def __enter__(self):
                    return self

                def __exit__(self, exc_type, exc_value, traceback):
                    pass

                def read(self):
                    # encode mock response
                    text = mock_read_response(url)
                    return text.encode('utf-8')

            return MockHttpResponse()

        # replace 'util.urlopen' with mock implementation
        util.urlopen = mock_urlopen



"""
Useful things for plotting GeoClaw results.

DEPRECATED - use the version in visclaw instead
"""

from clawpack.visclaw import colormaps
from matplotlib.colors import Normalize 
from clawpack.geoclaw import topotools
from numpy import ma
import warnings

w = """
*** WARNING: clawpack.geoclaw.geoplot is deprecated
             Use clawpack.visclaw.geoplot instead"""

warnings.warn(w, UserWarning)

# Colormaps from geoclaw
# Color attributes, single instance per run
# Colors
black = [0.0,0.0,0.0]
white = [1.0,1.0,1.0]
red = [1.0,0.0,0.0]
green = [0.0,1.0,0.0];
dark_green = [0.1,0.4,0.0];
light_green = [0.8,1.0,0.5];
blue = [0.0,0.0,1.0];
dark_blue = [0.2,0.2,0.7];
light_blue = [0.5,0.5,1.0];
blue_green = [0.0,1.0,1.0];
tan = [0.9,0.8,0.2];
tan = [0.8,0.5,0.2];
brown = [0.9,0.8,0.2];
gray8 = [0.8,0.8,0.8];
purple = [0.8,0.3,0.8];

# Colormaps
TSUNAMI_MAX_AMPLITUDE = 0.6
tsunami_colormap = colormaps.make_colormap({-TSUNAMI_MAX_AMPLITUDE:blue,
                                            0.0:blue_green,
                                            TSUNAMI_MAX_AMPLITUDE:red})
                                            
land1_colormap = colormaps.make_colormap({0.0:dark_green,
                                          1000.0:green,
                                          2000.0:light_green,
                                          4000.0:tan})
                                         
land2_colormap = colormaps.make_colormap({0:dark_green,
                                          50:green,
                                          100:light_green,
                                          200:tan})
                                          
water_land_colormap = colormaps.make_colormap({-1000:dark_blue,
                                               -500:blue,
                                               0:light_blue,
                                               .1:tan,
                                               5:tan,
                                               6:dark_green,
                                               1000:green,
                                               2000:light_green,
                                               4000:tan})
                                               
bathy1_colormap = colormaps.make_colormap({-1000:brown,
                                           0:tan,
                                           .1:dark_green,
                                           1000:green,
                                           2000:light_green})
       
bathy2_colormap = colormaps.make_colormap({-1000:brown,
                                           -100:tan,
                                           0:dark_green,
                                           .1:dark_green,
                                           1000:green,
                                           2000:light_green})

bathy3_colormap = colormaps.make_colormap({-1:[0.3,0.2,0.1],
                                           -0.01:[0.95,0.9,0.7],
                                           .01:[.5,.7,0],
                                           1:[.2,.5,.2]})

seafloor_colormap = colormaps.make_colormap({-1:[0.3,0.2,0.1],
                                              0:[0.95,0.9,0.7]})

land_colormap = colormaps.make_colormap({ 0:[0.95,0.9,0.7],
                                          1:[.2,.5,.2]})


                                           
colormaps_list = {"tsunami":tsunami_colormap,"land1":land1_colormap,
             "land2":land2_colormap,"water_land":water_land_colormap,
             "bathy1":bathy1_colormap,"bathy2":bathy2_colormap}
        
def plot_colormaps():
    r"""Plots all colormaps avaiable or the ones specified"""
        
    import numpy as np
    import matplotlib.pyplot as plt
    
    a = np.linspace(0, 1, 256).reshape(1,-1)
    a = np.vstack((a,a))
    
    nmaps = len(colormaps_list) + 1

    fig = plt.figure(figsize=(5,10))
    fig.subplots_adjust(top=0.99, bottom=0.01, left=0.2, right=0.99)
    
    for i,name in enumerate(colormaps_list):
        ax = plt.subplot(nmaps,1,i+1)
        plt.axis("off")
        plt.imshow(a, aspect='auto', cmap=colormaps_list[name], origin='lower')
        pos = list(ax.get_position().bounds)
        fig.text(pos[0] - 0.01, pos[1], name, fontsize=10, horizontalalignment='right')

    #plt.show()

land_colors = colormaps.make_colormap({0:[.5,.7,0], 1:[.2,.5,.2]})
# water_colors = colormaps.make_colormap({-1.:'r', 0.:[0, .8, .8], 1.: 'b'})
# land_colors = land2_colormap
water_colors = tsunami_colormap

# Plotting functions

# The drytol parameter is used in masking land and water and
# affects what color map is used for cells with small water depth h.
# The best value to use often depends on the application and can
# be set for an application by setting current_data.user.drytol in
# a beforeframe function, for example.  If it's not set by the user,
# the following default value is used (in meters):

drytol_default = 1.e-3


def topo(current_data):
   """ 
   Return topography = eta - h. 
   Surface eta is assumed to be output as 4th column of fort.q files.
   """
   q = current_data.q
   h = q[0,:,:]
   eta = q[3,:,:]
   topo = eta - h
   return topo

def land(current_data):
   """
   Return a masked array containing the surface elevation only in dry cells.
   """
   from numpy import ma
   drytol = getattr(current_data.user, 'drytol', drytol_default)
   q = current_data.q
   h = q[0,:,:]
   eta = q[3,:,:]
   land = ma.masked_where(h>drytol, eta)
   return land

def water(current_data):
   """Deprecated: use surface instead."""
   raise DeprecationWarning("Deprecated function, use surface instead.")
   from numpy import ma
   drytol = getattr(current_data.user, 'drytol', drytol_default)
   q = current_data.q
   h = q[0,:,:]
   eta = q[3,:,:]
   water = ma.masked_where(h<=drytol, eta)
   return water

def depth(current_data):
   """
   Return a masked array containing the depth of fluid only in wet cells.
   """
   from numpy import ma
   drytol = getattr(current_data.user, 'drytol', drytol_default)
   q = current_data.q
   h = q[0,:,:]
   depth = ma.masked_where(h<=drytol, h)
   return depth

def surface(current_data):
    """
    Return a masked array containing the surface elevation only in wet cells.
    Surface is eta = h+topo, assumed to be output as 4th column of fort.q
    files.
    """
    from numpy import ma
    drytol = getattr(current_data.user, 'drytol', drytol_default)
    q = current_data.q
    h = q[0,:,:]
    eta = q[3,:,:]
    water = ma.masked_where(h<=drytol, eta)
    return water

def surface_or_depth(current_data):
    """
    Return a masked array containing the surface elevation where the topo is 
    below sea level or the water depth where the topo is above sea level.
    Mask out dry cells.  Assumes sea level is at topo=0.
    Surface is eta = h+topo, assumed to be output as 4th column of fort.q
    files.
    """
    from numpy import ma, where
    drytol = getattr(current_data.user, 'drytol', drytol_default)
    q = current_data.q
    h = q[0,:,:]
    eta = q[3,:,:]
    topo = eta - h
    surface = ma.masked_where(h<=drytol, eta)
    depth = ma.masked_where(h<=drytol, h)
    surface_or_depth = where(topo<0, surface, depth)
    return surface_or_depth

# Some discrete color maps useful for contourf plots of fgmax results:

def discrete_cmap_1(clines):
    """
    Construct a discrete color map for the regions between the contour lines
    given in clines. Colors go from turqouise through yellow to red.
    Good for zeta.
    """
    from numpy import floor, linspace, hstack, ones, zeros
    nlines = len(clines)
    n1 = int(floor((nlines-1)/2.))
    n2 = nlines - 1 - n1
    Green = hstack([linspace(1,1,n1),linspace(1,0,n2)])
    Red = hstack([linspace(0,0.8,n1), ones(n2)])
    Blue = hstack([linspace(1,0.2,n1), zeros(n2)])
    colors = list(zip(Red,Green,Blue))
    return colors

def discrete_cmap_2(clines):
    """
    Construct a discrete color map for the regions between the contour lines
    given in clines.  Colors go from red to turquoise.
    Good for arrival times.
    """
    from numpy import floor, linspace, hstack, ones, zeros, flipud
    nlines = len(clines)
    n1 = int(floor((nlines-1)/2.))
    n2 = nlines - 1 - n1
    Green = flipud(hstack([linspace(1,1,n1),linspace(1,0,n2)]))
    Red = flipud(hstack([linspace(0,0.8,n1), ones(n2)]))
    Blue = flipud(hstack([linspace(1,0.2,n1), zeros(n2)]))
    colors = list(zip(Red,Green,Blue))
    return colors






from __future__ import absolute_import
def mapc2p(xc,yc):
    """
    Specifies the mapping to curvilinear coordinates -- should be consistent
    with mapc2p.f
    """
    from numpy import sin, cos

    # Polar coordinates (xc = r,  yc = theta)
    xp = xc * cos(yc)
    yp = xc * sin(yc)
    return xp,yp



"""
Plot fixed grid output.
fgno refers to the number of the fixed grid

Now creates html pages of plots by default if
executed from Unix command line via:
    $ python plotfg.py 1
to plot fixed grid number 1, for example.

Specify output directory other than _output by adding the outdir to this line:
    $ python plotfg.py 1 _output-05

"""

from pylab import *
from clawpack.visclaw import geoplot, colormaps, plotpages
# Cannot find this!
# from plottools import fix_long_tick_labels

import os
from numpy import ma
from clawpack.clawutil.data import ClawData


class ClawPlotFGData(ClawData):

    def __init__(self,fgno=1):
        super(ClawPlotFGData,self).__init__()
        self.add_attribute('fgno',fgno)
        self.add_attribute('drytol',1.e-2)
        self.add_attribute('exposed_tol',1.e-2)
        self.add_attribute('clines',linspace(-20,20,21))

        self.add_attribute('water_cmap', geoplot.tsunami_colormap)
        self.add_attribute('land_cmap', geoplot.land_colors)
        self.add_attribute('seafloor_cmap', geoplot.seafloor_colormap)
        self.add_attribute('inundated_cmap', \
             colormaps.make_colormap({0.:[1,.9,.9], 1.:[1,0,0]}))

        self.add_attribute('water_clim',(-1,1))
        self.add_attribute('land_clim',(0,10))
        self.add_attribute('seafloor_clim',(-1,0))
        self.add_attribute('inundated_clim',(0,1))

        self.add_attribute('water_add_colorbar',True)
        self.add_attribute('land_add_colorbar',False)
        self.add_attribute('seafloor_add_colorbar',True)
        self.add_attribute('inundated_add_colorbar',True)

        self.add_attribute('eta_show',True)
        self.add_attribute('seafloor_show',True)
        self.add_attribute('inundated_show',True)

        self.add_attribute('outdir','_output')
        self.add_attribute('plotdir',None)
        self.add_attribute('save_png',False)
        self.add_attribute('solutions',{})
        self.add_attribute('grids',{})
        self.add_attribute('combined_figure',True)

    def list_frames(self):

        import glob
        pattern = "%s/fort.fg%s_*" % (self.outdir,str(self.fgno).zfill(2))
        files = glob.glob(pattern)
        if len(files) == 0:
            print('*** No files found of form ', pattern)
        framenos = []
        for file in files:
            line = open(file,'r').readline()
            t = float(line.split()[0])
            print("%s: t = %s" % (file,t))
            frameno = file[-2:]
            framenos.append(int(frameno))
        return framenos


    def get_frame(self, frameno):

        if frameno in self.solutions:
            # don't read if already in dictionary:
            return self.grid, self.solutions[frameno]

        fname = "fort.fg%s_%s" % (str(self.fgno).zfill(2), str(frameno).zfill(4))
        fname = os.path.join(self.outdir,fname)
        if not os.path.exists(fname):
            print("*** Did not find file ",fname," in directory ",self.outdir)
            raise IOError("Missing fixed grid output file")
        

        self.grid = ClawData()
        grid = self.grid

        # Read parameters from header:

        file = open(fname,'r')

        line = file.readline()
        t = float(line.split()[0])
        print("Reading fixed grid output from ",fname)
        print('   Frame %s at t = %s' % (frameno,t))

        line = file.readline()
        grid.mx = int(line.split()[0])

        line = file.readline()
        grid.my = int(line.split()[0])

        line = file.readline()
        grid.xlow = float(line.split()[0])

        line = file.readline()
        grid.ylow = float(line.split()[0])

        line = file.readline()
        grid.xhi = float(line.split()[0])

        line = file.readline()
        grid.yhi = float(line.split()[0])

        file.close()

        grid.x = linspace(grid.xlow, grid.xhi, grid.mx+1)
        grid.y = linspace(grid.ylow, grid.yhi, grid.my+1)
        grid.dx = grid.x[1]-grid.x[0]
        grid.dy = grid.y[1]-grid.y[0]
        grid.xcenter = grid.x[:-1] + grid.dx/2.
        grid.ycenter = grid.y[:-1] + grid.dy/2.

        d = loadtxt(fname, skiprows=8)

        solution = ClawData()
        solution.t = t
        solution.ncols = d.shape[1]
        solution.h = reshape(d[:,0], (grid.my,grid.mx))
        solution.B = reshape(d[:,3], (grid.my,grid.mx))
        solution.eta = reshape(d[:,4], (grid.my,grid.mx))
        solution.surface = ma.masked_where(isnan(solution.eta),solution.eta)
        solution.land = ma.masked_where(solution.h>self.drytol,solution.B)
        solution.fg = empty((solution.ncols,grid.my,grid.mx), dtype=float)
        for col in range(solution.ncols):
            solution.fg[col,:,:] = reshape(d[:,col],(grid.my,grid.mx))
        
        self.solutions[frameno] = solution
        return grid, solution


    def plotfg(self, frameno):

        grid, solution = self.get_frame(frameno)
        print("Plotting frame %s at time t = %s"  % (frameno,solution.t))

    
        # Define function to plot topo contours for use in multiple places:
        def add_contours():
            contour(grid.xcenter,grid.ycenter,solution.B,self.clines,colors='k')
            contour(grid.xcenter,grid.ycenter,solution.B,[0.],colors='k',linewidths=2)
    
        #t_str = "%10.3f" % solution.t
        tmin = solution.t / 60.
        t_str = "%8.2f minutes" % tmin
    
        if self.combined_figure:
            figno = 153
            figure(figno, (14,8))
            clf()
            subplot(121)
        else:
            figno = 150
            figure(figno)
            clf()
        
        if ma.count(solution.surface) != 0:
            pcolormesh(grid.x,grid.y,solution.surface,cmap=self.water_cmap)
            clim(self.water_clim)
            if self.water_add_colorbar: colorbar(shrink=0.5)
        
        if ma.count(solution.land) != 0:
            pcolormesh(grid.x,grid.y,solution.land,cmap=self.land_cmap)
            clim(self.land_clim)
            if self.land_add_colorbar: colorbar(shrink=0.5)
        
        add_contours()
        
        
        title('Eta on FG %s at time = %s' % (self.fgno,t_str))
        xlim((grid.xlow, grid.xhi))
        ylim((grid.ylow, grid.yhi,))
        # fix_long_tick_labels()        
        axis('tight')
        axis('scaled')
    
        if self.save_png:
            fname = 'FixedGrid%sFrame%sfig%s.png' \
                %  (str(self.fgno).zfill(2), str(frameno).zfill(4), figno)
            savefig(fname)
            print("Saved figure as ",fname)
        
        
        if solution.ncols > 5:

            etamin = solution.fg[5,:,:]
            etamax = solution.fg[6,:,:]
    
            etamax2 = where(solution.B<0, 1., etamax)
            
            # Add red contour of maximum eta
            #contour(xcenter,ycenter,etamax2,[drytol],colors='r',linewidths=2)
            
            # Add brown contour of minimum eta
            #contour(xcenter,ycenter,etamin-B,[exposed_tol],colors=([.9,.8,.2],),linewidths=2)
    
            # Determine exposed and inundatated regions:
    
            B = solution.B
            exposed_tol = self.exposed_tol
            drytol = self.drytol
            exposed = ma.masked_where(((B>0) | (etamin > B+exposed_tol)), etamin)
            inundated = ma.masked_where(((B<0) | (etamax < B+drytol)), etamax-B)

        #---------------------------------------------------------------

        if self.combined_figure:
            subplot(122)

        if self.inundated_show:
            if solution.ncols < 7:
                raise ValueError("*** Data does not include etamin/etamax")
    
            if not self.combined_figure:
                figno = 151
                figure(figno)
                clf()
            
            if ma.count(inundated) != 0:
                pcolormesh(grid.x,grid.y,inundated,cmap=self.inundated_cmap)
                clim(self.inundated_clim)
                if self.inundated_add_colorbar: colorbar(shrink=0.5)
            add_contours()
            # Add red contour of maximum eta
            #contour(xcenter,ycenter,etamax2,[drytol],colors='r',linewidths=2)
            title("Inundated region for t <= %s" % t_str)
            xlim((grid.xlow, grid.xhi))
            ylim((grid.ylow, grid.yhi,))
            # fix_long_tick_labels()        
            axis('tight')
            axis('scaled')
            if self.save_png:
                fname = 'FixedGrid%sFrame%sfig%s.png' \
                    %  (str(self.fgno).zfill(2), str(frameno).zfill(4), figno)
                savefig(fname)
                print("Saved figure as ",fname)
        
        #---------------------------------------------------------------

        if self.seafloor_show:
            if solution.ncols < 7:
                raise ValueError("*** Data does not include etamin/etamax")
    
            if not self.combined_figure:
                figno = 152
                figure(figno)
                clf()
            
            if ma.count(exposed) != 0:
                pcolormesh(grid.x,grid.y,exposed,cmap=self.seafloor_cmap)
                clim(self.seafloor_clim)
                if self.seafloor_add_colorbar: colorbar(shrink=0.5)
            add_contours()
            # Add brown contour of minimum eta
            #contour(xcenter,ycenter,etamin-B,[exposed_tol],colors=([.9,.8,.2],),linewidths=2)
            if self.combined_figure:
                title("Eta min/max t <= %s" % t_str)
            else:
                title("Exposed seabed for t <= %s" % t_str)
            xlim((grid.xlow, grid.xhi))
            ylim((grid.ylow, grid.yhi,))
            # fix_long_tick_labels()        
            axis('tight')
            axis('scaled')
            if self.save_png:
                fname = 'FixedGrid%sFrame%sfig%s.png' \
                    %  (str(self.fgno).zfill(2), str(frameno).zfill(4), figno)
                savefig(fname)
                print("Saved figure as ",fname)
                
            
        
    def fgloop(self):
        for frameno in range(1,100):
            try:
                self.plotfg(frameno)
            except IOError:
                break
            ans = input("Hit return for next time, q to quit, s to savefig... ")
            if ans=='s':
                fname = 'FixedGrid%sFrame%s.png' %  (str(self.fgno).zfill(2), str(frameno).zfill(4))
                savefig(fname)
                print("Saved figure as ",fname)
                ans = input("Hit return for next time, q to quit, s to savefig... ")
                
            if ans=='q':
                break
    
                
    def fg2html(self,framenos='all'):
        if self.plotdir is None:
            self.plotdir='_fgplots_fg%s' % str(self.fgno).zfill(2)
        startdir = os.getcwd()
        self.outdir = os.path.abspath(self.outdir)
        plotpages.cd_with_mkdir(self.plotdir, overwrite=True)
        self.save_png = True

        ppd = plotpages.PlotPagesData()
        ppd.timeframes_frametimes = {}

        if framenos=='all':
            framenos = list(range(1,200))
    
        for frameno in framenos:
            try:
                grid, solution = self.get_frame(frameno)
                t = solution.t
                ppd.timeframes_frametimes[frameno] = t
                self.plotfg(frameno)
            except IOError:
                break
            draw()
    
        os.chdir(startdir)
        ppd.plotdir = self.plotdir
        ppd.html_index_fname = "_PlotIndex_FixedGrid%s.html" \
            % str(self.fgno).zfill(2)
        ppd.html_index_title = "Fixed Grids Plot Index"
        ppd.timeframes_prefix='FixedGrid%sFrame' % str(self.fgno).zfill(2)
        ppd.html_homelink = "_PlotIndex.html"
        ppd.timeframes_fignames[150] = 'Surface'
        ppd.timeframes_fignames[151] = 'Inundation'
        ppd.timeframes_fignames[152] = 'Exposed seafloor'
        plotpages.timeframes2html(ppd)
            


if __name__ == "__main__":
    # if executed at the Unix command line....
    import sys
    args = sys.argv[1:]   # any command line arguments
    fg = ClawFGData()
    if len(args) > 0:
        fg.fgno = args[0]
    if len(args) > 1:
        fg.outdir = args[1]
    fg.fg2html(fg)



"""
Compare gauges between two different runs, useful for regression testing.

Requires visclaw@c3bc5aab82 or later for gaugetools.compare_gauges.
"""

from __future__ import absolute_import
from __future__ import print_function
from clawpack.visclaw import gaugetools

outdir1 = '_output_original'
outdir2 = '_output'

tol = 0.  # tolerance for comparison
    
matches = gaugetools.compare_gauges(outdir1, outdir2, 
            gaugenos='all', q_components='all',
            tol=tol, verbose=True, plot=False)

if not matches:
    print("*** Warning: results to not all match to tol = %g" % tol)
    print("*** You might want to call gaugetools.compare_gauges with ")
    print("    plot=True to view differences.")





"""
Set up the domain and computational grid.
A piecewise linear topography is defined by specifying the topography `z`
value at a set of nodes `x` in the `xzpairs` list.

A nonuniform grid with `mx` grid cells is used with cell widths related
to the still water depth in such a way that the Courant number is roughly
constant in deep water and onto the shelf, and with uniform grid cells
near shore and onshore where the water depth is less than `hmin`.
"""

from pylab import *
from clawpack.geoclaw import nonuniform_grid_tools

x1 = -300e3
x2 = 150e3

def makegrid(xs):

    xzpairs = [( x1, -3200),   # left edge
               (-xs, -3200),   # start of continental slope
               ( xs,  -200),    # start of continental shelf
               ( x2,  -200)]    # right edge

    topo_fcn = nonuniform_grid_tools.make_pwlin_topo_fcn(xzpairs)

    mx = 5000   # number of grid cells
    hmin = 50.  # mininum depth for varying cell widths

    nonuniform_grid_tools.make_celledges_cfl(x1, x2, mx, topo_fcn,
            hmin=hmin, fname='celledges.data', plot_topo=True)

if __name__=='__main__':
    #makegrid(xs=0.e3)
    #makegrid(xs=15.e3)
    makegrid(xs=60.e3)



"""
Plot timing info found in timing.csv file.
Requires modified valout function from Clawpack 5.5.0 to print this info.

To use, first run the Clawpack or GeoClaw code, and insure that the 
file `timing.csv` was generated in the output directory.
This file has one line for each output time giving the cumulative CPU and
Wall time, both in total and broken down by levels.

If you execute this at the command line you can specify the output
directory, e.g.

    python plot_timing_stat.py _output

and plots will be made with some default choices of units and png files
placed in the output directory.

For more control, import this module and call the function `make_plots`.
This allows setting units and also redirecting the png files to the _plots
directory and creating an html file to easily view them.
"""

from __future__ import print_function
from pylab import *
import os

# text for html file showing all plots:

html_text1 = """
    <html>
    <h1>Timing information</h1>
    <p>
    <pre>
    """
html_text2 = """
    </pre>
    <p>
    <img width=30% src="timing_CumWallTime.png">
    <img width=30% src="timing_CumCPUTime.png">
    <img width=30% src="timing_CumCellUpdates.png">
    
    <p>
    <img width=30% src="timing_ByFrameWallTime.png">
    <img width=30% src="timing_ByFrameCPUTime.png">
    <img width=30% src="timing_ByFrameCellUpdates.png">
    
    <p>
    <img width=50% src="timing_ByFrameCellUpdatesPerCPU.png">
    
    </html>
"""

#=======================================================================

def make_plots(outdir='_output', make_pngs=True, make_html=None, 
               plotdir=None, units={}):

    """
    `outdir` is the directory containing `timing.csv`.
    Set `make_pngs` to `True` to create png files, 
    By default, an html index will be made if `make_pngs == True` but
    you can turn this off by setting `make_html` to `False`.
    If `plotdir == None` then png file will be put in `outdir`.
    
    The `units` directory can be used to choose units for the plots:

    units['comptime'] can be 'seconds', 'minutes', 'hours', or 'days'.
        Since computer time (wall and CPU) is in seconds in the csv file,
        changing these units will rescale by a factor of 60, 3600, or
        24*3600 in the plots (for 'minutes', 'hours', or 'days' respectively).
        The default is 'seconds' if you do not set this element of the 
        dictionary.

    units['simtime'] is the units used in the simulation.  
        The default is 'nondimensional', which is appropriate if the PDE is 
        non-dimensionalized or there are no specific units.

    You can set units['simtime'] to an arbitrary string to be used in labels
        on the axes, if this is the units of the simulation.  If you want to 
        scale `t` by a multiplicative factor then you can also specify
        units['simtime_factor'].  For example, if `t` in the PDE is
        in nanoseconds but you ran out so long that you want to the units 
        in the plots to be milliseconds, you could set:
            units['simtime'] = 'milliseconds'
            units['simtime_factor'] = 1e6
        Then `t` will be divided by `1e6` before plotting.

    Special case: Since many simulations use units in which `t` is in seconds 
        (in particular, GeoClaw simulations): 

        if units['simtime'] one of:
             'seconds', 'minutes', 'hours', or 'days',
        and if units['simtime_factor'] is not specified, 
        then these factors default to:
             1, 60, 3600, or 24*3600 respectively.
        So if `t` in your PDE is in hours and you want to plot as such, 
        you will have to over-ride this default with
            units['simtime'] = 'hours'
            units['simtime_factor'] = 1.

    
    units['cell'] is the units used in plotting the number of grid cells
        updated between output frames, and the cumulative plot.

        if units['cell'] == 'raw count' then number of cells is plotted
        if units['cell'] is one of 'thousands', 'millions', 'billions', or
            'trillions', then the number of cell updates is divided by the
            appropriate factor (1e3, 1e6, 1e9, 1e12 resp.) before plotting.
        if you want to use some other units, you can also explicitly set
            units['cell_factor'] to the appropriate value.
        The default is units['cell'] == 'millions', appropriate for many
            smallish problems.
        

    """

    if make_pngs:
        if plotdir is None:
            plotdir = outdir
        os.system('mkdir -p %s' % plotdir)

    if make_html is None:
        make_html = make_pngs

    if make_html and (not make_pngs):
        print('Warning: will make html index but no png files')

    def make_png(fname):
        if make_pngs:
            tight_layout() # since labels sometimes disappear
            fname = os.path.join(plotdir, fname)
            savefig(fname)
            #savefig(fname, bbox_inches='tight')
            print('Created %s' % fname)

    # set desired units for simulation time and computer time,
    # based on length of run:

    # set units and scaling factors for Simulation time t (simtime), 
    # CPU/Wall time (comptime), and for number of cells updated.
    # units is used for text in plots, factor is used for scaling values read in
    # if None, then no units appear in plots, and/or no scaling is done.

    # computer time is recorded in seconds so this is the natural units:
    #comptime_units = 'seconds'
    #comptime_units = 'minutes'
    
    comptime_units = units.get('comptime', 'seconds')
    if comptime_units == 'seconds':
        comptime_factor = 1
    elif comptime_units == 'minutes':
        comptime_factor = 60.
    elif comptime_units == 'hours':
        comptime_factor = 3600.
    elif comptime_units == 'days':
        comptime_factor = 24*3600.
    else:
        comptime_units = 'dimensionless'
        comptime_factor = 1

    simtime_units = units.get('simtime', 'dimensionless')
    simtime_factor = units.get('simtime_factor', 'default')

    # note: if units['simtime_factor'] is not specified it is set to 'default',
    # which causes special case behavior below.

    if simtime_units == 'dimensionless':
        simtime_factor = 1
    if simtime_units == 'seconds':
        if simtime_factor == 'default':
            simtime_factor = 1
    elif simtime_units == 'minutes':
        if simtime_factor == 'default':
            simtime_factor = 60.
    elif simtime_units == 'hours':
        if simtime_factor == 'default':
            simtime_factor = 3600.
    elif simtime_units == 'days':
        if simtime_factor == 'default':
            simtime_factor = 24*3600.
    elif simtime_factor == 'default':
        print("*** warning, no default simtime_factor for units['simtime'] = %s" \
              % simtime_units)
        print("Using simtime_factor = 1")
        simtime_factor = 1

    # Units for number of cell updates:
    cell_units = units.get('cell', 'millions')
    cell_factor = units.get('cell_factor', 'default')

    if cell_units == 'raw count':
        cell_factor = 1
    if cell_units == 'thousands':
        cell_factor = 1e3
    if cell_units == 'millions':
        cell_factor = 1e6
    elif cell_units == 'billions':
        cell_factor = 1e9
    elif cell_units == 'trillions':
        cell_factor = 1e12
    else:
        if cell_factor is 'default':
            print("*** Warning, units['cell'] = %s has no default cell_factor, using 1" \
             % cell_units)
            cell_factor = 1


    # define colors, with colors[0] used for overhead, colors[j] for level j >= 1
    colors = ['gray'] + 3*['r','c','m','limegreen','b','orange','g','yellow'] 


    # Read in timing states from output directory:
    timing_stats_file = os.path.join(outdir, 'timing.csv')
    timing_stats = loadtxt(timing_stats_file, skiprows=1, delimiter=',',
                           ndmin=2)
    ntimes = timing_stats.shape[0]
    nlevels = int(timing_stats.shape[1] / 3) - 1

    time = zeros(ntimes)
    total_cpu = zeros(ntimes)
    total_wall = zeros(ntimes)
    wtime = zeros((ntimes, nlevels))
    cpu = zeros((ntimes, nlevels))
    cells = zeros((ntimes, nlevels))

    for j in range(ntimes):
        time[j] = timing_stats[j,0] / simtime_factor
        total_wall[j] = timing_stats[j,1] / comptime_factor
        total_cpu[j] = timing_stats[j,2] / comptime_factor
        for level in range(nlevels):
            wtime[j, level] = timing_stats[j,3*level + 3] / comptime_factor
            cpu[j, level] = timing_stats[j,3*level + 4] / comptime_factor
            cells[j, level] = timing_stats[j,3*level + 5] / cell_factor
        
    xlimits = [time.min(), time.max()]
    ylimits_comptime = [0, 1.1*total_cpu.max()]
    ylimits_cells = [0, 1.1*sum(cells[-1,:])]


    figure(21)
    clf()
    sum_cells_over_levels = zeros(ntimes)
    for j in range(nlevels):
        if max(cells[:,j]) == 0:
            break
        #plot(time/3600, cells[:,j], label='Level %s' % (j+1))
        last_sum_cells = sum_cells_over_levels.copy()
        sum_cells_over_levels += cells[:,j]
        plot(time, sum_cells_over_levels, 'k')
        fill_between(time, last_sum_cells, sum_cells_over_levels, 
                     color=colors[j+1], edgecolor=None,
                     label='Level %s' % (j+1))

    plot(time, sum_cells_over_levels, 'k', lw=1, label='Total Cells')
    xlim(xlimits)
    #ylim(0, 1.1*sum_cells_over_levels[-1])
    ylim(ylimits_cells)
    title('Cumulative cells updated on each level')
    xlabel('Simulation time t (%s)' % simtime_units)
    if cell_units is None:
        ylabel('Grid cells updated')
    else:
        ylabel('Grid cells updated (%s)' % cell_units)

    legend(loc='upper left')

    make_png('timing_CumCellUpdates.png')


    figure(22)
    clf()
    sum_cpu_over_levels = zeros(ntimes)
    for j in range(nlevels):
        if max(cpu[:,j]) == 0:
            break
        #plot(time/3600, cpu[:,j], label='Level %s' % (j+1))
        last_sum_cpu = sum_cpu_over_levels.copy()
        sum_cpu_over_levels += cpu[:,j]
        fill_between(time, last_sum_cpu, sum_cpu_over_levels, color=colors[j+1],
                      edgecolor=None,label='Level %s' % (j+1))
        plot(time, sum_cpu_over_levels, 'k')

    fill_between(time, total_cpu,sum_cpu_over_levels,color=colors[0],
                  edgecolor=None,label='Overhead')
    plot(time, total_cpu, 'k', lw=1, label='Total CPU')
    xlim(xlimits)
    ylim(ylimits_comptime)
    title('Cumulative CPU time on each level')
    xlabel('Simulation time t (%s)' % simtime_units)
    ylabel('CPU time (%s)' % comptime_units)
    legend(loc='upper left')

    make_png('timing_CumCPUTime.png')


    figure(23)
    clf()
    sum_wtime_over_levels = zeros(ntimes)
    for j in range(nlevels):
        if max(wtime[:,j]) == 0:
            break
        last_sum_wtime = sum_wtime_over_levels.copy()
        sum_wtime_over_levels += wtime[:,j]
        fill_between(time, last_sum_wtime,
                     sum_wtime_over_levels, 
                     color=colors[j+1], edgecolor=None,
                     label='Level %s' % (j+1))
        plot(time, sum_wtime_over_levels, 'k')

    fill_between(time, total_wall, sum_wtime_over_levels, color=colors[0],
                  edgecolor=None,label='Overhead')
    plot(time, total_wall, 'k', lw=1, label='Total Wall')
    title('Cumulative wall time on each level')
    xlabel('Simulation time t (%s)' % simtime_units)
    ylabel('Wall time (%s)' % comptime_units)
    legend(loc='upper left')
    xlim(xlimits)
    ylim(ylimits_comptime)

    make_png('timing_CumWallTime.png')


    # cells between frames:

    figure(31)
    clf()
    dc_max = 0
    for n in range(1,ntimes):
        dt = (time[n] - time[n-1])
        tt = array([time[n-1],time[n]])
        if dt == 0:
            continue
        dcn = 0
        for j in range(nlevels):
            #dc = (cells[n,j] - cells[n-1,j]) / dt
            dc = (cells[n,j] - cells[n-1,j])
            if dc >= 0:
                plot(tt, [dcn+dc, dcn+dc], 'k')
                if n == 1:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc],
                                 color=colors[j+1], edgecolor=None,
                                 label='Level %s' % (j+1))
                else:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc],
                                  edgecolor=None,color=colors[j+1])
            dcn = dcn + dc

        plot([time[n-1],time[n-1]], [0,dcn], 'k')
        plot([time[n],time[n]], [0,dcn], 'k')
        dc_max = max(dc_max, dcn)

                             
    xlim(xlimits)
    ylim(0, 1.2*dc_max)
    title('Cells updated between output frames')
    xlabel('Simulation time t (%s)' % simtime_units)
    if cell_units is None:
        ylabel('Grid cells updated')
    else:
        ylabel('Grid cells updated (%s)' % cell_units)

    legend(loc='upper left')

    make_png('timing_ByFrameCellUpdates.png')


    # cpu_time between frames:

    figure(32)
    clf()
    dc_max = 0
    for n in range(1,ntimes):
        dt = (time[n] - time[n-1])
        tt = array([time[n-1],time[n]])
        if dt == 0:
            continue
        dcn = 0
        for j in range(nlevels):
            dc = (cpu[n,j] - cpu[n-1,j]) #/ dt
            if dc >= 0:
                plot(tt, [dcn+dc, dcn+dc], 'k')
                if n == 1:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc],
                                 color=colors[j+1], edgecolor=None,
                                 label='Level %s' % (j+1))
                else:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc],
                                  edgecolor=None,color=colors[j+1])
            dcn = dcn + dc

        if n == 1:
            kwargs_label = {'label': 'Overhead'}
        else:
            kwargs_label = {}
        dtot = (total_cpu[n]-total_cpu[n-1]) #/ dt 
        plot(tt, [dtot,dtot], 'k')
        fill_between(tt, [dcn,dcn], [dtot,dtot], 
                     color=colors[0],
                     alpha=1, edgecolors='k', **kwargs_label)

        plot([time[n-1],time[n-1]], [0,dtot], 'k')
        plot([time[n],time[n]], [0,dtot], 'k')
        
        dc_max = max(dc_max, dtot)
         
        #plot(time, total_cpu, 'k', lw=1, label='Total CPU')

                             
    #plot(time, sum_cells_over_levels, 'k', lw=1, label='Total Cells')
    xlim(xlimits)
    ylimits_avecomptime = [0, 1.2*dc_max]
    ylim(ylimits_avecomptime)

    title('CPU time between output frames')
    xlabel('Simulation time t (%s)' % simtime_units)
    ylabel('CPU time (%s) / frame' % comptime_units)
    legend(loc='upper left')

    make_png('timing_ByFrameCPUTime.png')



    # wall time between frames:

    figure(33)
    clf()
    dc_max = 0
    for n in range(1,ntimes):
        dt = (time[n] - time[n-1])
        tt = array([time[n-1],time[n]])
        if dt == 0:
            continue
        dcn = 0
        for j in range(nlevels):
            #last_dc = last_dc + dc
            dc = (wtime[n,j] - wtime[n-1,j]) #/ dt
            if dc >= 0:
                plot(tt, [dcn+dc, dcn+dc], 'k')
                if n == 1:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc],
                                 color=colors[j+1], edgecolor=None,
                                 label='Level %s' % (j+1))
                else:
                    fill_between(tt, [dcn,dcn], [dcn+dc,dcn+dc], edgecolor=None,
                                 color=colors[j+1])
            dcn = dcn + dc

        if n == 1:
            kwargs_label = {'label': 'Overhead'}
        else:
            kwargs_label = {}
        dtot = (total_wall[n]-total_wall[n-1]) #/ dt 
        plot(tt, [dtot,dtot], 'k')
        fill_between(tt, [dcn,dcn], [dtot,dtot], 
                     color=colors[0],
                     alpha=1, edgecolors='k', **kwargs_label)

        plot([time[n-1],time[n-1]], [0,dtot], 'k')
        plot([time[n],time[n]], [0,dtot], 'k')
        
        dc_max = max(dc_max, dtot)

                             
    xlim(xlimits)
    #ylim(0, 1.2*dc_max)
    ylim(ylimits_avecomptime)
    title('Wall time between output frames')
    xlabel('Simulation time t (%s)' % simtime_units)
    ylabel('Wall time (%s)/ output frame' % comptime_units)
    legend(loc='upper left')

    make_png('timing_ByFrameWallTime.png')




    # average cells updates per CPU second between frames:

    figure(41)
    clf()
    dc_max = 0
    for n in range(1,ntimes):
        dt = (time[n] - time[n-1])
        tt = array([time[n-1],time[n]])
        if dt == 0:
            continue



        total_cpu_frame = sum(cpu[n,0:nlevels])
        total_cells_frame = sum(cells[n,0:nlevels])
        cells_cpu = total_cells_frame / total_cpu_frame
        fill_between(tt, [cells_cpu, cells_cpu], color='c')
        plot(tt, [cells_cpu, cells_cpu], 'k')
        plot([time[n-1],time[n-1]], [0,cells_cpu], 'k')
        plot([time[n],time[n]], [0,cells_cpu], 'k')
        
        #dc_max = max(dc_max, dcn)
        dc_max = max(dc_max, cells_cpu)

                             
    xlim(xlimits)
    ylim(0, 1.2*dc_max)
    title('Average cells updated per CPU time, between output frames')
    xlabel('Simulation time t (%s)' % simtime_units)
    if cell_units in [None,'raw count']:
        ylabel('Grid cells updated / CPU (%s)' % comptime_units)
    else:
        ylabel('Grid cells updated (%s) / CPU (%s)' % (cell_units,comptime_units))

    #legend(loc='upper left')

    make_png('timing_ByFrameCellUpdatesPerCPU.png')

    if make_pngs:

        try:
            import datetime
            timing_text = open(outdir+'/timing.txt','r').read()
            date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')
            timing_text = "<h4>Datetime: %s   From: %s\n</h4>" \
                          % (date,os.path.abspath(outdir)) \
                          + timing_text
        except:
            timing_text = 'Error -- could not read timing.txt from %s' \
                           % outdir
        html_text = html_text1 + timing_text + html_text2

    if make_html:
        html_file = os.path.join(plotdir, 'timing.html')
        with open(html_file,'w') as h:
            h.write(html_text)
        print('Created %s' % html_file)


if __name__=='__main__':
    import sys
    print(sys.argv)
    if len(sys.argv) > 1:
        outdir = sys.argv[1]
    else:
        outdir = '_output'
    make_plots(outdir)



#!/usr/bin/env python

# How to use this file
# python setup.py build_ext -i

import os
from os.path import join as pjoin
from os.path import pardir as pardir

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('advection_reaction_2d', parent_package, top_path)

    config.add_extension('advection_2d',
                         ['rpn2_vc_advection_vector.f90', 'rpt2_dummy.f90'])


    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python
# encoding: utf-8
r"""
Routines for reading and writing a HDF5 output file

This module reads and writes hdf5 files via either of the following modules:
    h5py - http://code.google.com/p/h5py/
    PyTables - http://www.pytables.org/moin

It will first try h5py and then PyTables and use the correct calls
according to whichever is present on the system.  We recommend that you use
h5py as it is a minimal wrapper to the HDF5 library.

To install either, you must also install the hdf5 library from the website:
    http://www.hdfgroup.org/HDF5/release/obtain5.html
"""

import os
import logging

from clawpack import pyclaw
import numpy as np

logger = logging.getLogger('pyclaw.fileio')

# Import appropriate hdf5 package
use_h5py = False
use_PyTables = False
try:
    import h5py
    use_h5py = True
except:
    try:
        import tables
        use_PyTables = True
    except:
        logging.critical("Could not import h5py or PyTables!")
        error_msg = ("Could not import h5py or PyTables, please install " +
            "either h5py or PyTables.  See the doc_string for more " +
            "information.")
        raise Exception(error_msg)


def write(solution,frame,path,file_prefix='claw',write_aux=False,
                options={},write_p=False):
    r"""
    Write out a Solution to a HDF5 file.
    
    :Input:
     - *solution* - (:class:`~pyclaw.solution.Solution`) Pyclaw solution 
       object to input into
     - *frame* - (int) Frame number
     - *path* - (string) Root path
     - *file_prefix* - (string) Prefix for the file name.  ``default = 'claw'``
     - *write_aux* - (bool) Boolean controlling whether the associated 
       auxiliary array should be written out.  ``default = False``     
     - *options* - (dict) Optional argument dictionary, see 
       `HDF5 Option Table`_
    
    .. _`HDF5 Option Table`:
    
    +-----------------+------------------------------------------------------+
    | Key             | Value                                                |
    +=================+======================================================+
    | compression     | (None, string ["gzip" | "lzf" | "szip"] or int 0-9)  |
    |                 | Enable dataset compression. DEFLATE, LZF and (where  |
    |                 | available) SZIP are supported. An integer is         |
    |                 | interpreted as a GZIP level for backwards            |
    |                 | compatibility.                                       |
    +-----------------+------------------------------------------------------+
    |compression_opts | (None, or special value) Setting for compression     |
    |                 | filter; legal values for each filter type are:       |
    |                 |                                                      |
    |                 | - *gzip* - (int) 0-9                                 |
    |                 | - *lzf* - None allowed                               |
    |                 | - *szip* - (tuple) 2-tuple ('ec'|'nn', even integer  |
    |                 |     0-32)                                            |
    |                 |                                                      |
    |                 | See the filters module for a detailed description of |
    |                 | each of these filters.                               |
    +-----------------+------------------------------------------------------+
    | chunks          | (None, True or shape tuple) Store the dataset in     |
    |                 | chunked format. Automatically selected if any of the |
    |                 | other keyword options are given. If you don't provide|
    |                 | a shape tuple, the library will guess one for you.   |
    +-----------------+------------------------------------------------------+
    | shuffle         | (True/False) Enable/disable data shuffling, which can|
    |                 | improve compression performance. Automatically       |
    |                 | enabled when compression is used.                    |
    +-----------------+------------------------------------------------------+
    | fletcher32      | (True/False) Enable Fletcher32 error detection; may  |
    |                 | be used with or without compression.                 |
    +-----------------+------------------------------------------------------+
    """
    option_defaults = {'compression':None,'compression_opts':None,
                       'chunks':None,'shuffle':False,'fletcher32':False}
    for (k,v) in option_defaults.items():
        options[k] = options.get(k,v)
    
    filename = os.path.join(path,'%s%s.hdf' % 
                                (file_prefix,str(frame).zfill(4)))
    
    if use_h5py:
        with h5py.File(filename,'w') as f:
        
            # For each patch, write out attributes
            for state in solution.states:
                patch = state.patch
                # Create group for this patch
                subgroup = f.create_group('patch%s' % patch.patch_index)

                # General patch properties
                subgroup.attrs['t'] = state.t
                subgroup.attrs['num_eqn'] = state.num_eqn
                subgroup.attrs['num_aux'] = state.num_aux
                for attr in ['num_ghost','patch_index','level']:
                    if hasattr(patch,attr):
                        if getattr(patch,attr) is not None:
                            subgroup.attrs[attr] = getattr(patch,attr)

                # Add the dimension names as a attribute
                subgroup.attrs['dimensions'] = [name.encode('utf-8') 
                            for name in patch.get_dim_attribute('name')]
                # Dimension properties
                for dim in patch.dimensions:
                    for attr in ['num_cells','lower','delta','upper',
                                 'units']:
                        if hasattr(dim,attr):
                            if getattr(dim,attr) is not None:
                                attr_name = '%s.%s' % (dim.name,attr)
                                subgroup.attrs[attr_name] = getattr(dim,attr)

                if write_p:
                    q = state.p
                else:
                    q = state.q
                subgroup.create_dataset('q',data=q,**options)
                if write_aux and state.num_aux > 0:
                    subgroup.create_dataset('aux',data=state.aux,**options)
        
    elif use_PyTables:
        # f = tables.openFile(filename, mode = "w", title = options['title'])
        logging.critical("PyTables has not been implemented yet.")
        raise IOError("PyTables has not been implemented yet.")
    else:
        err_msg = "No hdf5 python modules available."
        logging.critical(err_msg)
        raise Exception(err_msg)

def read(solution,frame,path='./',file_prefix='claw',read_aux=True,
                options={}):
    r"""
    Read in a HDF5 file into a Solution
    
    :Input:
     - *solution* - (:class:`~pyclaw.solution.Solution`) Pyclaw object to be 
       output
     - *frame* - (int) Frame number
     - *path* - (string) Root path
     - *file_prefix* - (string) Prefix for the file name.  ``default = 'claw'``
     - *write_aux* - (bool) Boolean controlling whether the associated 
       auxiliary array should be written out.  ``default = False``     
     - *options* - (dict) Optional argument dictionary, not used for reading.
    """
    filename = os.path.join(path,'%s%s.hdf' % 
                                (file_prefix,str(frame).zfill(4)))
    patches = []

    if use_h5py:
        with h5py.File(filename,'r') as f:
        
            for patch in f.values():
                # Construct each dimension
                dimensions = []
                dim_names = np.array(patch.attrs["dimensions"]).astype(str)
                for dim_name in dim_names:
                    dim = pyclaw.solution.Dimension(
                                        patch.attrs["%s.lower" % dim_name],
                                        patch.attrs["%s.upper" % dim_name],
                                        patch.attrs["%s.num_cells" % dim_name],
                                        name = dim_name)
                    # Optional attributes
                    for attr in ['units']:
                        attr_name = "%s.%s" % (dim_name,attr)
                        if patch.attrs.get(attr_name, None):
                            setattr(dim,attr,patch.attrs["%s.%s" % (dim_name,attr)])
                    dimensions.append(dim)

                pyclaw_patch = pyclaw.solution.Patch(dimensions)

                # Fetch general patch properties
                for attr in ['t','num_eqn','patch_index','level']:
                    setattr(pyclaw_patch,attr,patch.attrs[attr])

                state = pyclaw.state.State(pyclaw_patch, \
                         patch.attrs['num_eqn'],patch.attrs['num_aux'])
                state.t = patch.attrs['t']
                state.q = patch['q'][:].ravel(order='F').reshape(state.q.shape,order='F')

                # Read in aux if applicable
                if read_aux and patch.get('aux',None) is not None:
                    state.aux = patch['aux'][:].ravel(order='F').reshape(state.aux.shape,order='F')

                solution.states.append(state)
                patches.append(pyclaw_patch)
                
            solution.domain = pyclaw.geometry.Domain(patches)
                
    elif use_PyTables:
        # f = tables.openFile(filename, mode = "r", title = options['title'])
        logging.critical("PyTables has not been implemented yet.")
        raise IOError("PyTables has not been implemented yet.")
    else:
        err_msg = "No hdf5 python modules available."
        logging.critical(err_msg)
        raise Exception(err_msg)



#!/usr/bin/env python
# encoding: utf-8
r"""
2D shallow water: radial dam break
==================================

Solve the 2D shallow water equations:

.. math::
    h_t + (hu)_x + (hv)_y = 0 \\
    (hu)_t + (hu^2 + \frac{1}{2}gh^2)_x + (huv)_y = 0 \\
    (hv)_t + (huv)_x + (hv^2 + \frac{1}{2}gh^2)_y = 0.

The initial condition is a circular area with high depth surrounded by lower-depth water.
The top and right boundary conditions reflect, while the bottom and left boundaries
are outflow.
"""

import numpy as np
from clawpack import riemann
from clawpack.riemann.shallow_roe_with_efix_2D_constants import depth, x_momentum, y_momentum, num_eqn

def qinit(state,h_in=2.,h_out=1.,dam_radius=0.5):
    x0=0.
    y0=0.
    X, Y = state.p_centers
    r = np.sqrt((X-x0)**2 + (Y-y0)**2)

    state.q[depth     ,:,:] = h_in*(r<=dam_radius) + h_out*(r>dam_radius)
    state.q[x_momentum,:,:] = 0.
    state.q[y_momentum,:,:] = 0.

    
def setup(kernel_language='Fortran', use_petsc=False, outdir='./_output',
          solver_type='classic', riemann_solver='roe',disable_output=False):
    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if riemann_solver.lower() == 'roe':
        rs = riemann.shallow_roe_with_efix_2D
    elif riemann_solver.lower() == 'hlle':
        rs = riemann.shallow_hlle_2D

    if solver_type == 'classic':
        solver = pyclaw.ClawSolver2D(rs)
        solver.limiters = pyclaw.limiters.tvd.MC
        solver.dimensional_split=1
    elif solver_type == 'sharpclaw':
        solver = pyclaw.SharpClawSolver2D(rs)

    solver.bc_lower[0] = pyclaw.BC.extrap
    solver.bc_upper[0] = pyclaw.BC.wall
    solver.bc_lower[1] = pyclaw.BC.extrap
    solver.bc_upper[1] = pyclaw.BC.wall

    # Domain:
    xlower = -2.5
    xupper = 2.5
    mx = 150
    ylower = -2.5
    yupper = 2.5
    my = 150
    x = pyclaw.Dimension(xlower,xupper,mx,name='x')
    y = pyclaw.Dimension(ylower,yupper,my,name='y')
    domain = pyclaw.Domain([x,y])

    state = pyclaw.State(domain,num_eqn)

    # Gravitational constant
    state.problem_data['grav'] = 1.0

    qinit(state)

    claw = pyclaw.Controller()
    claw.tfinal = 2.5
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    if disable_output:
        claw.output_format = None
    claw.outdir = outdir
    claw.num_output_times = 10
    claw.setplot = setplot
    claw.keep_copy = True

    return claw

#--------------------------
def setplot(plotdata):
#--------------------------
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for depth
    plotfigure = plotdata.new_plotfigure(name='Water height', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-2.5, 2.5]
    plotaxes.ylimits = [-2.5, 2.5]
    plotaxes.title = 'Water height'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.red_yellow_blue
    plotitem.pcolor_cmin = 0.5
    plotitem.pcolor_cmax = 1.5
    plotitem.add_colorbar = True
    
    # Scatter plot of depth
    plotfigure = plotdata.new_plotfigure(name='Scatter plot of h', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0., 2.5]
    plotaxes.ylimits = [0., 2.1]
    plotaxes.title = 'Scatter plot of h'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_from_2d_data')
    plotitem.plot_var = depth
    def q_vs_radius(current_data):
        from numpy import sqrt
        x = current_data.x
        y = current_data.y
        r = sqrt(x**2 + y**2)
        q = current_data.q[depth,:,:]
        return r,q
    plotitem.map_2d_to_1d = q_vs_radius
    plotitem.plotstyle = 'o'


    # Figure for x-momentum
    plotfigure = plotdata.new_plotfigure(name='Momentum in x direction', figno=2)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-2.5, 2.5]
    plotaxes.ylimits = [-2.5, 2.5]
    plotaxes.title = 'Momentum in x direction'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = x_momentum
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True
    plotitem.show = False       # show on plot?
    

    # Figure for y-momentum
    plotfigure = plotdata.new_plotfigure(name='Momentum in y direction', figno=3)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-2.5, 2.5]
    plotaxes.ylimits = [-2.5, 2.5]
    plotaxes.title = 'Momentum in y direction'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = y_momentum
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True
    plotitem.show = False       # show on plot?
    
    return plotdata


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



r"""
Module for the CFL object, which is responsible for computing and enforcing the
Courant-Friedrichs-Lewy condition.
"""

class CFL(object):
    def __init__(self, global_max):
        self._global_max = global_max
        
    def get_global_max(self):
        r"""
        Compute the maximum CFL number over all processes for the current step.

        This is used to determine whether the CFL condition was
        violated and adjust the timestep.
        """
        return self._global_max

    def get_cached_max(self):
        return self._global_max

    def set_local_max(self,new_local_max):
        self._global_max = new_local_max

    def set_global_max(self,new_global_max):
        self._global_max = new_global_max

    def update_global_max(self,new_local_max):
        self._global_max = max(self._global_max,new_local_max)



"""
This script reads constants from comments in the Fortran Riemann solvers and
writes the values to miniature modules so they can be imported and used in
Python scripts.  The constants that are read and written include:

- num_eqn
- num_waves
- names/indices of q fields
- names/indices of aux fields
"""
import os

for filename in os.listdir('.'):
    # Only process normal Riemann solver files
    if filename.startswith('rp1') or filename.startswith('rpn'):
        f = open(filename,'r')
        q_indices = []; q_names = []
        aux_indices = []; aux_names = []
        q_block   = False
        aux_block = False
        num_aux = None
        num_eqn = None
        # Read variable names and other constants
        for line in f.readlines():
            if len(line.split())<2:
                q_block   = False
                aux_block = False
            elif q_block:
                q_indices.append(int(line.split()[-2])-1)
                q_names.append(line.split()[-1])
            elif aux_block:
                aux_indices.append(int(line.split()[-2])-1)
                aux_names.append(line.split()[-1])
            elif 'waves:' in line.lower():
                num_waves = int(line.split()[-1])
            elif 'equations:' in line.lower():
                num_eqn = int(line.split()[-1])
            elif 'aux fields:' in line.lower():
                num_aux = int(line.split()[-1])
            elif 'conserved quantities:' in line.lower():
                q_block = True
            elif 'auxiliary variables:' in line.lower():
                aux_block = True
        f.close()

        if num_eqn is None: # No information from comments
            continue
        # Write constants and variable names to file
        if filename.startswith('rp1'):
            out_name = filename.split('.')[0][4:]+'_1D_constants.py'
        elif filename.startswith('rpn2'):
            out_name = filename.split('.')[0][5:]+'_2D_constants.py'
        elif filename.startswith('rpn3'):
            out_name = filename.split('.')[0][5:]+'_3D_constants.py'
        out_name = './'+out_name
        print('writing ' + out_name)
        f = open(out_name,'w')
        f.write('num_waves = '+str(num_waves)+'\n')
        f.write('num_eqn   = '+str(num_eqn)+'\n')
        if num_aux is not None:
            f.write('num_aux   = '+str(num_aux)+'\n')
        f.write('\n')
        f.write('# Conserved quantities\n')
        for variable, index in zip(q_names,q_indices):
            f.write(variable + ' = ' + str(index) + '\n')
        f.write('\n')
        if len(aux_names)>0:
            f.write('# Auxiliary variables\n')
        for variable, index in zip(aux_names,aux_indices):
            f.write(variable + ' = ' + str(index) + '\n')
        f.close()
    else:
        pass




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

from __future__ import absolute_import
import numpy
from clawpack.geoclaw import dtopotools

dtopo1 = dtopotools.DTopography('dtopo1.tt3',3)
dtopo2 = dtopotools.DTopography('dtopo2.tt3',3)

#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from clawpack.visclaw import colormaps, geoplot

    plotdata.clearfigures()  # clear any old figures,axes,items data

    # To plot gauge locations on pcolor or contour plot, use this as
    # an afteraxis function:

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos='all', format_string='ko', add_labels=True)


    #-----------------------------------------
    # Figure for pcolor plot
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='pcolor', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.title = 'Surface'
    plotaxes.scaled = True

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.show = True
    plotitem.plot_var = geoplot.surface
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -5.
    plotitem.pcolor_cmax = 5.
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    #plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.pcolor_cmin = 1.0
    plotitem.pcolor_cmax = 6.
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1
    plotaxes.xlimits = [-1,1]
    plotaxes.ylimits = [-1,1]

    # Add contour lines of bathymetry:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.show = True
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    #plotitem.contour_levels = [-14,-13,-12,-11]

    # need to adjust if h0 is changed in maketopo.py:
    plotitem.contour_levels = linspace(-3000,-1100,21)

    plotitem.amr_contour_colors = ['g']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [1,0]  
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0

    plotaxes.afteraxes = addgauges

    #-----------------------------------------
    # Figure for cross section
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='cross-section', figno=1)
    #plotfigure.show = False

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-1,1]
    plotaxes.ylimits = [-5,7]
    y0 = 0.0
    plotaxes.title = 'Cross section at y=%s' % y0

    plotitem = plotaxes.new_plotitem(plot_type='1d_from_2d_data')

    def xsec(current_data):
        # Return x value and surface eta at this point, along y=0
        from pylab import where,ravel
        x = current_data.x
        y = ravel(current_data.y)
        dy = current_data.dy
        q = current_data.q

        ij = where((y <= y0+dy/2.+1e-8) & (y > y0-dy/2.))[0]
        x_slice = ravel(x)[ij]
        eta_slice = ravel(q[3,:,:])[ij]
        return x_slice, eta_slice

    plotitem.map_2d_to_1d = xsec
    plotitem.plotstyle = 'o'
    plotitem.kwargs = {'markersize':5}
    plotitem.amr_show = [1]  # plot on all levels

    def add_dtopo_plot(current_data):
        from pylab import where, plot, legend
        for dtopo in [dtopo1,dtopo2]:
            j = max(where(dtopo.y <= y0)[0])
            j1 = min(j+1, len(dtopo.y)-1)
            beta = (y0 - dtopo.y[j])/(dtopo.y[1] - dtopo.y[0])
    
            t = current_data.t
            itlist = where(dtopo.times <= t)[0]
            if len(itlist) > 0:
                it = max(where(dtopo.times <= t)[0])
                #print "+++ t, it, dtopo.times[it]: ",t, it, dtopo.times[it]
                dz = dtopo.dZ[it,:,:]
                dzj = dz[j,:]
                if it < len(dtopo.times)-1:
                    dz1 = dtopo.dZ[it+1,:,:]
                    dz1j = dz1[j,:]
                    alpha = (t - dtopo.times[it])/(dtopo.times[it+1] - dtopo.times[it])
                    #print "+++ alpha = ",alpha
                    dzj = (1.-alpha)*dzj + alpha*dz1j
                plot(dtopo.x, dzj, label="dtopo")
        legend()
    plotaxes.afteraxes = add_dtopo_plot


    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface at gauges', figno=300, \
                    type='each_gauge')
    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(211)'
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Surface'

    # Plot surface as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 3
    plotitem.plotstyle = 'bo-'

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(212)'
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Topography'

    # Plot topo as green curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False

    def gaugetopo(current_data):
        q = current_data.q
        h = q[0,:]
        eta = q[3,:]
        topo = eta - h
        return topo

    plotitem.plot_var = gaugetopo
    plotitem.plotstyle = 'g-'


    #-----------------------------------------
    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

    



#!/usr/bin/env python
# encoding: utf-8

"""Tests for reading and writing storm data"""

from __future__ import absolute_import
from __future__ import print_function

import tempfile
import shutil
import os
import sys
import datetime

import numpy

import clawpack.clawutil.test as test
import clawpack.geoclaw.surge.storm as storm

# Set local test directory to get local files
testdir = os.path.dirname(__file__)
if len(testdir) == 0:
    testdir = "./"

# Current tests
file_format_tests = ['atcf', 'hurdat', 'jma', 'tcvitals', 'ibtracs']

def check_geoclaw(paths, check_header=False):
    """Check that two geoclaw formatted storm files are identical

    This does not use object equivalence due to round off errors that can occur
    due to format constraints.  If the *check_header* is True the routine also
    checks that the number of lines is equivalent (is implicitly checked
    anyway) but more importantly that the *time_offset*s are equivalent.

    :Input:
     - *check_header* (bool) Check that the headers in the file are equivalent.
       Defaults to `False`.

    :Raises:
     - *AssertionError* - If the files do not agree to the precision required.

    """

    if check_header:
        with open(paths[0], 'r') as data_file[0], open(paths[1], 'r') as data_file[1]:
            # Check for number of lines
            assert(int(data_file[0].readline()) == int(data_file[1].readline()))

            # Check for time offset
            assert(data_file[0].readline() == data_file[1].readline())

    # Check rest of data
    data = []
    for path in paths:
        data.append(numpy.loadtxt(path, skiprows=3))
    numpy.testing.assert_almost_equal(data[0], data[1])

# TODO - turn this into a test generator for each file IO format rather than
#        a single loop
def test_storm_IO(save=False):
    r"""Test reading and writing of storm formats


    Currently this only tests reading in data in all formats save for IMD and
    writing them out in the geoclaw format.  This functionality will be added
    once full writing functionality for the other formats is complete.

    :Input:
     - *save* (list or bool) whether to save the data produced by the test as
       new test data.  This can either be a single `bool` that will be applied
       to all formats or a dictionary that should have keys for each format.  If
       a format is not included in the dict than it is assumed `False`.

    """

    save_dict = {}
    if isinstance(save, bool):
        for key in file_format_tests:
            save_dict[key] = save
    elif isinstance(save, dict):
        for key in file_format_tests:
            save_dict[key] = save.get(key, default=False)
    else:
        raise ValueError("Type %s is not valid for save argument." % type(save))

    # Create temp directory
    temp_path = tempfile.mkdtemp()

    try:
        # Currently we read in the format, write it back out in the GeoClaw
        # format and check the stored GeoClaw file for that format
        for file_format in file_format_tests:
            if file_format=='ibtracs':
                file_suffix = 'nc'
                # Check here to see if we have xarray
                try:
                    import xarray
                except ImportError as e:
                    print("Skipping IBTrACS IO test, missing xarray.")
                    continue
            elif file_format == 'atcf':
                file_suffix = 'txt'
                # Check here to see if we have pandas
                try:
                    import pandas
                except ImportError as e:
                    print("Skipping ATCF IO test, missing pandas.")
                    continue
            else:
                file_suffix = 'txt'
            input_path = os.path.join(testdir, "data", "storm", "%s.%s" % (file_format,file_suffix))
            out_path = os.path.join(temp_path, '%s_geoclaw.txt' % file_format)
            check_path = os.path.join(testdir, "data", "storm",
                                      "%s_geoclaw.txt" % file_format)

            # Read in test data and write it back out in the GeoClaw format
            # for IBTrACS input, need storm/year info
            if file_format=='ibtracs':
                # test for Ike using EITHER storm_name and year OR
                # sid
#                 kwargs = {'storm_name':'IKE',
#                          'year':2008}
                kwargs = {'sid': '2008245N17323',
                          'agency_pref': ['wmo','usa']}

                # test the fill_radius_w_other_source func
                atcf_path = os.path.join(testdir, "data", "storm", "atcf.txt")
                storm_atcf = storm.Storm(atcf_path, file_format='ATCF')
                def fill_mwr(t, this_storm):
                    return storm.fill_rad_w_other_source(t, this_storm, storm_atcf, 'max_wind_radius')
                def fill_rad(t, this_storm):
                    return storm.fill_rad_w_other_source(t, this_storm, storm_atcf, 'storm_radius')
            else:
                kwargs = {}
                fill_mwr = None
                fill_rad = None
            test_storm = storm.Storm(input_path, file_format=file_format, **kwargs)

            # Temporary testing thing to get around missing data in formats that
            # do not provide the proper radii
            if file_format in ['hurdat', 'jma']:
                test_storm.max_wind_radius[:] = 0.0
                test_storm.storm_radius[:] = 0.0

            test_storm.write(out_path, file_format="geoclaw",
                             max_wind_radius_fill = fill_mwr,
                             storm_radius_fill = fill_rad)

            # Save new geoclaw test files into check_path if requested
            if save:
                test_storm.write(check_path, file_format="geoclaw",
                             max_wind_radius_fill = fill_mwr,
                             storm_radius_fill = fill_rad)

            # Check geoclaw files
            check_geoclaw([out_path, check_path])

    except Exception as e:
        # If the assertion failed then copy the contents of the directory
        test_dump_path = os.path.join(os.getcwd(), 'test_storm_IO')
        if os.path.exists(test_dump_path):
            shutil.rmtree(test_dump_path)
        shutil.copytree(temp_path, os.path.join(os.getcwd(),
                                                'test_storm_IO'))
        print("Format test %s -> geoclaw  failed." % file_format)
        raise e

    finally:
        shutil.rmtree(temp_path)


if __name__ == '__main__':
    # Currently does not support only saving one of the format's data
    save = False
    if len(sys.argv) > 1:
        save = bool(sys.argv[1])
    test_storm_IO(save)



"""Main petclaw package"""

from __future__ import absolute_import
import os
import logging, logging.config

# Default logging configuration file
_DEFAULT_LOG_CONFIG_PATH = os.path.join(os.path.dirname(__file__),'../pyclaw/log.config')
del os

# Setup loggers
logging.config.fileConfig(_DEFAULT_LOG_CONFIG_PATH)

__all__ = []

# Module imports
__all__.extend(['Controller','Dimension','Patch','Domain','Solution','State','CFL','riemann'])
from .controller import Controller
from clawpack.petclaw.geometry import Patch, Domain 
from clawpack.pyclaw.geometry import Dimension
from .solution import Solution
from .state import State
from .cfl import CFL

__all__.extend(['ClawSolver1D','ClawSolver2D','ClawSolver3D','SharpClawSolver1D','SharpClawSolver2D','SharpClawSolver3D'])
from .classic.solver import ClawSolver1D,ClawSolver2D,ClawSolver3D
from .sharpclaw.solver import SharpClawSolver1D,SharpClawSolver2D,SharpClawSolver3D

__all__.append('BC')
from clawpack.pyclaw.solver import BC

# Sub-packages
from . import limiters
from .limiters import *
__all__.extend(limiters.__all__)

from . import plot
__all__.append('plot')



#!/usr/bin/env python
# encoding: utf-8
r"""
Advection-reaction in 2D
========================
Solve the 2D advection-reaction problem

.. math::
    p_t + u(x,y,t) p_x + v(x,y,t) p_y = \epsilon q \\
    q_t + u(x,y,t) q_x + v(x,y,t) q_y = \epsilon p

Note that the left hand side of this system is the non-conservative transport
equation for p and q.  The Riemann solver assumes that velocities are specified
at cell edges.

This example also shows how to use your own Riemann solver.
"""

import numpy as np
import os

try:
    from clawpack.pyclaw.examples.advection_reaction_2d import advection_2d
except ImportError:
    this_dir = os.path.dirname(__file__)
    if this_dir == '':
        this_dir = os.path.abspath('.')
    import subprocess
    cmd = "make -C "+this_dir
    proc = subprocess.Popen("make -C "+this_dir+"/", shell=True, stdout = subprocess.PIPE)
    proc.wait()
    try:
        # Now try to import again
        from clawpack.pyclaw.examples.advection_reaction_2d import advection_2d
    except ImportError:
        raise



t_period = 4.0
epsilon = 0.1

def source_step(solver,state,dt):
    "Integrate the source term over one step."
    qq = state.q.copy()
    state.q[0,:,:] = state.q[0,:,:] + epsilon*dt*qq[1,:,:]
    state.q[1,:,:] = state.q[1,:,:] + epsilon*dt*qq[0,:,:]


def psi(x,y):
    "Velocity field potential."
    return (np.sin(np.pi*x))**2 * (np.sin(np.pi*y))**2 / np.pi

def set_velocities(solver,state):
    "Update velocity field for current time."
    v_t = np.cos(2 * np.pi * (state.t + solver.dt/2) / t_period)
    X, Y = state.grid.p_nodes
    dx, dy = state.grid.delta
    # u(x_(i-1/2),y_j)
    state.aux[0,:,:] = - ( psi(X[:-1,1:],Y[:-1,1:]) - psi(X[:-1,:-1],Y[:-1,:-1]) ) / dy
    # v(x_i,y_(j-1/2))
    state.aux[1,:,:] =   ( psi(X[1:,:-1],Y[1:,:-1]) - psi(X[:-1,:-1],Y[:-1,:-1]) ) / dx
    state.aux[:] = v_t * state.aux[:]


def setup(outdir='./_output'):
    from clawpack import pyclaw
    from clawpack.pyclaw.examples.advection_reaction_2d import advection_2d

    solver = pyclaw.ClawSolver2D(advection_2d)
    # Use dimensional splitting since no transverse solver is defined
    solver.dimensional_split = 1

    solver.all_bcs = pyclaw.BC.extrap
    solver.aux_bc_lower[0] = pyclaw.BC.extrap
    solver.aux_bc_upper[0] = pyclaw.BC.extrap
    solver.aux_bc_lower[1] = pyclaw.BC.extrap
    solver.aux_bc_upper[1] = pyclaw.BC.extrap

    domain = pyclaw.Domain( (0.,0.), (1.,1.), (100,100) )
    solver.num_eqn = 2
    solver.num_waves = 1
    num_aux = 2
    state = pyclaw.State(domain, solver.num_eqn, num_aux)

    Xe, Ye = domain.grid.p_nodes
    Xc, Yc = domain.grid.p_centers
    dx, dy = domain.grid.delta

    # Edge velocities
    # u(x_(i-1/2),y_j)
    state.aux[0,:,:] = - ( psi(Xe[:-1,1:],Ye[:-1,1:]) - psi(Xe[:-1,:-1],Ye[:-1,:-1]) ) / dy
    # v(x_i,y_(j-1/2))
    state.aux[1,:,:] =   ( psi(Xe[1:,:-1],Ye[1:,:-1]) - psi(Xe[:-1,:-1],Ye[:-1,:-1]) ) / dx

    solver.before_step = set_velocities
    solver.step_source = source_step
    solver.source_split = 1

    state.q[0,:,:] = (Xc <= 0.5)
    state.q[1,:,:] = (Yc <= 0.5)

    claw = pyclaw.Controller()
    claw.tfinal = t_period
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.keep_copy = True
    claw.outdir = outdir
    if outdir == '':
        claw.output_format = None
    claw.setplot = setplot

    return claw

def setplot(plotdata):
    """ 
    Plot solution using VisClaw.
    """ 
    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for pcolor plot
    plotfigure = plotdata.new_plotfigure(name='q[0]', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'q[0]'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 1.0
    plotitem.add_colorbar = True
    
    # Figure for contour plot
    plotfigure = plotdata.new_plotfigure(name='q[1]', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'q[1]'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 1
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 1.0
    plotitem.add_colorbar = True
    
    return plotdata

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python

r"""Bowl-Slosh regression test for GeoClaw

To create new regression data use
    `python regression_tests.py True`
"""

from __future__ import absolute_import
from __future__ import print_function
import os
import sys
import unittest
import subprocess
import tempfile
import time

import numpy
import xarray as xr
import nose

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools

build_failure_str = ("NetCDF topography test skipped due to failure to build "
                     "test program.")
class NetCDFBowlSloshTest(test.GeoClawRegressionTest):

    r"""NetCDF regression test for GeoClaw based on the bowl-slosh example"""

    def __init__(self, methodName="runTest"):

        super(NetCDFBowlSloshTest, self).__init__(methodName=methodName)


    def setUp(self):

        self.temp_path = tempfile.mkdtemp()

        self.stdout = open(os.path.join(self.temp_path, "run_output.txt"), "w")
        self.stdout.write("Output from Test %s\n" % self.__class__.__name__)
        # TODO - Should change this to use the time module's formatting
        # apparatus
        tm = time.localtime()
        year = str(tm[0]).zfill(4)
        month = str(tm[1]).zfill(2)
        day = str(tm[2]).zfill(2)
        hour = str(tm[3]).zfill(2)
        minute = str(tm[4]).zfill(2)
        second = str(tm[5]).zfill(2)
        date = 'Started %s/%s/%s-%s:%s.%s\n' % (year,month,day,hour,minute,second)
        self.stdout.write(date)
        self.stdout.write(("="*80 + "\n"))

        self.stderr = open(os.path.join(self.temp_path, "error_output.txt"), "w")
        self.stderr.write("Errors from Test %s\n" % self.__class__.__name__)
        self.stderr.write(date)
        self.stderr.write(("="*80 + "\n"))

        self.stdout.flush()
        self.stderr.flush()

        self.stdout.write("Paths:")
        self.stdout.write("  %s" % self.temp_path)
        self.stdout.write("  %s" % self.test_path)
        self.stdout.flush()

        # Make topography
        a = 1.
        h0 = 0.1
        z = lambda x,y: h0 * (x**2 + y**2) / a**2 - h0

        topo = topotools.Topography(topo_func=z)
        topo.x = numpy.linspace(-3.1, 3.1, 310)
        topo.y = numpy.linspace(-3.5,2.5, 300)

        try:
            import netCDF4
            this_path = os.path.join(self.temp_path, 'bowl.nc')

            # now mess with the order of the dimension IDs (lat, then lon)
            with netCDF4.Dataset(this_path,'w') as out:
                lat = out.createDimension('lat',len(topo.y))
                lon = out.createDimension('lon',len(topo.x))

                # create latitude first
                latitudes = out.createVariable('lat','f8',("lat",))
                longitudes = out.createVariable('lon','f8',("lon",))
                elevations = out.createVariable('elevation','f8',("lat","lon"))
                latitudes[:] = topo.y
                longitudes[:] = topo.x
                elevations[:] = topo.Z


        except ImportError:
            # Assume that NetCDF is not installed and move on
            raise nose.SkipTest(build_failure_str)

        except RuntimeError as e:
            print(e.message)
            raise nose.SkipTest("NetCDF topography test skipped due to " +
                                "runtime failure.")
        else:
            self.build_executable()

    def build_executable(self):
        try:
            self.stdout.write("Teting NetCDF output:\n")
            subprocess.check_call("cd %s ; make netcdf_test " % self.test_path,
                                                                stdout=self.stdout,
                                                                stderr=self.stderr,
                                                                shell=True)
        except subprocess.CalledProcessError:

            self.stdout.write(build_failure_str)
            raise nose.SkipTest(build_failure_str)

        else:
            # Force recompilation of topo_module to add NetCDF flags
            mod_path = os.path.join(os.environ["CLAW"], "geoclaw", "src", "2d",
                                    "shallow", "topo_module.mod")
            obj_path = os.path.join(os.environ["CLAW"], "geoclaw", "src", "2d",
                                    "shallow", "topo_module.o")
            if os.path.exists(mod_path):
                os.remove(mod_path)
            if os.path.exists(obj_path):
                os.remove(obj_path)

            super(NetCDFBowlSloshTest, self).build_executable()


    def runTest(self, save=False, indices=(2, 3)):
        r"""Test NetCDF topography support

        """

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, indices=(2, 3),
                          tolerance=1e-4)


    def tearDown(self):
        r"""Tear down test infrastructure.

        This version removes source that may have been compiled with NetCDF
        turned on so that it does not conflict with subsequent tests.

        """

        # Force recompilation of topo_module to add NetCDF flags
        mod_path = os.path.join(os.environ["CLAW"], "geoclaw", "src", "2d",
                                "shallow", "topo_module.mod")
        obj_path = os.path.join(os.environ["CLAW"], "geoclaw", "src", "2d",
                                "shallow", "topo_module.o")
        if os.path.exists(mod_path):
            os.remove(mod_path)
        if os.path.exists(obj_path):
            os.remove(obj_path)

        super(NetCDFBowlSloshTest, self).tearDown()



if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = NetCDFBowlSloshTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



from . import advection_2d
import numpy as np
import os

thisdir = os.path.dirname(__file__)
path_expected_sols = os.path.join(thisdir,'expected_sols.npy')
expected_sols_dict = np.load(path_expected_sols,allow_pickle=True).item()

def error(test_name,**kwargs):
    claw = advection_2d.setup(outdir=None,**kwargs)
    claw.run()
    qtest = claw.frames[claw.num_output_times].state.get_q_global().reshape([-1])
    dx = claw.solution.domain.grid.delta[0]
    dy = claw.solution.domain.grid.delta[1]
    assert test_name in expected_sols_dict.keys(), f"Test name {test_name} not found in {path_expected_sols}"
    qexpected = expected_sols_dict[test_name]
    diff_L1 = dx*dy*np.sum(np.abs(qtest-qexpected))
    return diff_L1

class TestAdvection2D:
    def test_classic_dimensional_split(self):
        assert error(test_name='classic_dim_split', solver_type='classic',
                         dimensional_split=True)<1e-6

    def test_classic_unsplit_no_trans(self):
        assert error(test_name='classic_unsplit_no_trans', solver_type='classic',
                         dimensional_split=False,transverse_waves=0)<1e-6

    def test_classic_unsplit_trans_inc(self):
        assert error(test_name='classic_unsplit_trans_inc',solver_type='classic',
                         dimensional_split=False,transverse_waves=1)<1e-6
        
    def test_classic_unsplit_trans_cor(self):
        assert error(test_name='classic_unsplit_trans_cor', solver_type='classic',
                         dimensional_split=False,transverse_waves=2)<1e-6
    
    def test_sharpclaw(self):
        assert error(test_name='sharpclaw', solver_type='sharpclaw')<1e-6



#!/usr/bin/env python
r"""
Consolidate common and custom sources into one source list.

If a custom file has the same name as a common file, the common one is excluded.
Exclusions can also be manually set for replacement files with different names.

NOTE:  Unfortunately we may need to implement something that allows flexibility
as to when a module is compiled and available due to depedency issues
"""

from __future__ import print_function

from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import warnings
import argparse


def check_duplicate_fortran_src(src_list):
    r"""Check to see if there may be hidden fixed-format Fortran files.

    """

    output = ""

    # Extract all base paths to check for duplicate src conflicts
    base_paths = []
    for src_file in src_list:
        if os.path.dirname(src_file) not in base_paths:
            base_paths.append(os.path.dirname(src_file))

    for src_file in src_list:
        base_name = os.path.splitext(os.path.basename(src_file))[0]
        extension = os.path.splitext(src_file)[-1]

        if extension.lower() in (".f90", ".f"):
            if extension.lower() == ".f90":
                file_name = "%s.f" % base_name
            elif extension.lower() == ".f":
                file_name = "%s.f90" % base_name

            for base_path in base_paths:
                check_file = os.path.join(base_path, file_name)
                if os.path.exists(check_file):
                    output = "".join((output, "%s -- %s, " % (src_file,
                                                              check_file)))

    return output


def consolidate_src_lists(src_files, common_src_files, excluded_src_files):
    r"""
    Takes source lists provided and consolidates them based on the following:
     - source in the *src_files* list is always added,
     - source in the *common_src_files* list is included except if it is in the
       *exclued_src_files* list or specified in *src_files*.
    """

    excluded_src_names = [os.path.basename(os.path.splitext(src_file)[0])
                          for src_file in excluded_src_files] +  \
                         [os.path.basename(os.path.splitext(src_file)[0])
                          for src_file in src_files]

    src_list = []
    for src_file in common_src_files:
        name = os.path.basename(os.path.splitext(src_file)[0])
        if name not in excluded_src_names:
            src_list.append(src_file)

    for src_file in src_files:
        src_list.append(src_file)

    return src_list


def parse_args(arg_list, delimiter=";"):
    r"""Parses the command line input into lists separated by *delimiter*
    """
    file_lists = [[]]

    for arg in arg_list:
        if arg == delimiter:
            file_lists.append([])
        else:
            file_lists[-1].append(arg)

    return file_lists


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Consolidate and check Fortran source list.")
        print("  Usage:  ./check_src.py [consolidate|conflict] ...")
        print("  Two functions exist in this script: ")
        print("    (1) consolidate - a unified list of source based on three")
        print("        lists, the overall source list, a list of source that")
        print("        be included if not already in the first source list or")
        print("        if it is in the third exclude list.")
        print("    (2) conflict - Given a list of source files check to see")
        print("        if there might be Fortran 90 source with '.f90' that")
        print("        would hide a fixed-format Fortran source with '.f'.")

    if sys.argv[1].lower() == "consolidate":
        source_files, common_files, excluded_files = parse_args(sys.argv[2:])
        src_list = consolidate_src_lists(source_files, common_files,
                                                       excluded_files)
        print(" ".join(src_list))

    elif sys.argv[1].lower() == "conflicts":
        print(check_duplicate_fortran_src(sys.argv[2:]))

    else:
        raise ValueError("ERROR:  Unknown sub-command %s." % sys.argv[1])



#!/usr/bin/env python

r"""Bowl-Slosh regression test for GeoClaw

To create new regression data use
    `python regression_tests.py True`
"""

from __future__ import absolute_import
import os
import sys
import unittest

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools


class BowlSloshTest(test.GeoClawRegressionTest):

    r"""Bowl-Slosh regression test for GeoClaw"""

    def setUp(self):

        super(BowlSloshTest, self).setUp()

        # Make topography
        a = 1.
        h0 = 0.1
        topo_func = lambda x,y: h0 * (x**2 + y**2) / a**2 - h0

        topo = topotools.Topography(topo_func=topo_func)
        topo.topo_type = 2
        topo.x = numpy.linspace(-2.0, 2.0, 200)
        topo.y = numpy.linspace(-2.0, 2.0, 200)
        topo.write(os.path.join(self.temp_path, "bowl.topotype2"), \
                topo_type=2, Z_format="%22.15e")

        # fgmax_grids.data created by setrun.py now contains all info
        #from . import make_fgmax_grid 
        #make_fgmax_grid.make_fgmax_grid1(self.temp_path)


    def runTest(self, save=False, indices=(2, 3)):
        r"""Test bowl-slosh example

        Note that this stub really only runs the code and performs no tests.

        """

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, indices=(2, 3))
        self.check_fgmax(save=save)
        self.success = True


if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = BowlSloshTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
r"""
Test suite for forestclaw
"""

from __future__ import absolute_import

from .geometry import Patch
from clawpack.pyclaw.geometry import Dimension


def test_forestclaw_patch():
    """Test the simple extension of the pyclaw.Patch class"""

    patch = Patch(Dimension(0.0, 1.0, 10))
    patch.block_number = 2
    patch.mpi_rank = 3


if __name__ == '__main__':
    import nose
    nose.main()



#!/usr/bin/env python
# encoding: utf-8
r""" 
3D shock-bubble interaction problem.
A planar shock wave impacts a spherical region of low density.

This problem involves the 3D Euler equations:
.. math::
    \rho_t + (\rho u)_x + (\rho v)_y + (\rho w)_z & = 0 \\
    (\rho u)_t + (\rho u^2 + p)_x + (\rho uv)_y & = 0 \\
    (\rho v)_t + (\rho uv)_x + (\rho v^2 + p)_y + (\rho vw)_z & = 0 \\
    (\rho w)_t + (\rho uw)_x + (\rho vw)_y + (\rho w^2 + p)_z & = 0 \\
    E_t + \nabla \cdot (u (E + p) ) & = 0.


The conserved quantities are: 
    density (rho), x-,y-, and z-momentum (rho*u,rho*v,rho*w), and energy.
"""
import numpy as np
from scipy import integrate

gamma = 1.4 # Ratio of Specific Heats
gamma1 = gamma - 1.
x0 = 0.5; y0 = 0.; z0 = 0. # Bubble location
r_bubble = 0.2 # Bubble radius

# Ambient state
rhoout = 1.0
pout   = 1.0

# Bubble state
rhoin  = 0.1
pin    = 1.0

xshock = 0.2 # Initial shock wave location

# State behind shock wave
p_shock = 5.0
rho_shock = (gamma1 + p_shock*(gamma+1.))/ ((gamma+1.) + gamma1*p_shock)
v_shock = (p_shock - 1.) / np.sqrt(0.5 * ((gamma+1.) * p_shock+gamma1))
e_shock = 0.5*rho_shock*v_shock**2 + p_shock/gamma1


def bubble(y, x, zdown, zup, which):
    "Used to compute how much of each cell is in the bubble."
    def sphere_top(y, x, which):
        z2 = r_bubble**2 - (x-x0)**2 - (y-y0)**2
        if z2 < 0:
            return 0
        else:
            return z0 + np.sqrt(z2)

    def sphere_bottom(y, x, which):
        z2 = (r_bubble**2 - (x-x0)**2 - (y-y0)**2)
        if z2 < 0:
            return 0
        else:
            return z0 - np.sqrt(z2)

    top = min(sphere_top(y,x, which), zup)
    bottom = min(top,max(sphere_bottom(y,x, which), zdown))
    return top-bottom

def incoming_shock(state,dim,t,qbc,auxbc,num_ghost):
    """
    Incoming shock at x=0 boundary.
    """
    for i in range(num_ghost):
        qbc[0,i,...] = rho_shock
        qbc[1,i,...] = rho_shock*v_shock
        qbc[2,i,...] = 0.
        qbc[3,i,...] = 0.
        qbc[4,i,...] = e_shock


def setup(kernel_language='Fortran', solver_type='classic', use_petsc=False,
          dimensional_split=False, outdir='_output', output_format='hdf5',
          disable_output=False, num_cells=(256,64,64),
          tfinal=0.6, num_output_times=10):

    from clawpack import riemann
    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='classic':
        solver = pyclaw.ClawSolver3D(riemann.euler_3D)
        solver.dimensional_split = dimensional_split
    else:
        raise Exception('Unrecognized solver_type.')

    x = pyclaw.Dimension(0.0, 2.0, num_cells[0], name='x')
    y = pyclaw.Dimension(0.0, 0.5, num_cells[1], name='y')
    z = pyclaw.Dimension(0.0, 0.5, num_cells[2], name='z')
    domain = pyclaw.Domain([x,y,z])

    solver.all_bcs = pyclaw.BC.extrap
    solver.bc_lower[0]  = pyclaw.BC.custom
    solver.user_bc_lower = incoming_shock
    solver.bc_lower[1] = pyclaw.BC.wall
    solver.bc_lower[2] = pyclaw.BC.wall

    state = pyclaw.State(domain,solver.num_eqn)

    state.problem_data['gamma']  = gamma
    
    grid = state.grid
    X,Y,Z = grid.p_centers
    r0 = np.sqrt((X-x0)**2 + (Y-y0)**2 + (Z-z0)**2)

    state.q[0,:,:,:] = rho_shock*(X<xshock) + rhoout*(X>=xshock) # density (rho)
    state.q[1,:,:,:] = rho_shock*v_shock*(X<xshock) # x-momentum (rho*u)
    state.q[2,:,:,:] = 0. # y-momentum (rho*v)
    state.q[3,:,:,:] = 0. # z-momentum (rho*w)
    state.q[4,:,:,:] = e_shock*(X<xshock) + pout/gamma1*(X>=xshock) # energy (e)

    # Compute cell fraction inside bubble
    dx, dy, dz = state.grid.delta
    dx2, dy2, dz2 = [d/2. for d in state.grid.delta]
    dmax = max(state.grid.delta)

    for i in range(state.q.shape[1]):
        for j in range(state.q.shape[2]):
            for k in range(state.q.shape[3]):
                if (r0[i,j,k] - dmax > r_bubble):
                    continue
                xdown = X[i,j,k] - dx2
                xup   = X[i,j,k] + dx2
                ydown = lambda x : Y[i,j,k] - dy2
                yup   = lambda x : Y[i,j,k] + dy2
                zdown = Z[i,j,k] - dz2
                zup   = Z[i,j,k] + dz2

                infrac,abserr = integrate.dblquad(bubble,xdown,xup,ydown,yup,
                                                  args=(zdown,zup,0),
                                                  epsabs=1.e-3,epsrel=1.e-2)
                infrac=infrac/(dx*dy*dz)

                state.q[0,i,j,k] = rhoout*(1.-infrac) + rhoin*infrac
                state.q[4,i,j,k] = (pout*(1.-infrac) + pin*infrac)/gamma1 # energy (e)

    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.output_format = output_format
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None
    claw.tfinal = tfinal
    claw.num_output_times = num_output_times
    claw.outdir = outdir

    return claw

# __main__()
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup)



from __future__ import absolute_import
from __future__ import print_function
from numpy import *
from FRS import vec_rp
q = random.random((2,800))
aux = random.random((2,800))
waves, s = vec_rp(2,q[:,0:799],q[:,1:800],aux[:,0:799],aux[:,1:800])
print(waves)
print(s)



